#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <utility.h>
#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include "life.h"

typedef struct {
    unsigned nextGenNumNeighbors:4;
    unsigned numNeighbors:4;
    unsigned isAlive:1;
    unsigned stateChangedSinceLastDisplay:1;
    unsigned reserved:22;
} CellType;

static int panelHandle, popupPanel, ringPanel;

#define kMaxTableBufSize    302
#define kMaxNumSavedFiles   200
#define kSettingsFile       "life.set"
#define kLifeFileFmtStr     "life%04d.lif" 
#define kLifeFilePnlStates  "life.pnl" 

/* dynamically allocated in main */
static CellType **table;

static char saveTable[kMaxTableBufSize][kMaxTableBufSize];

static char projDir[MAX_PATHNAME_LEN];

static int tableSize; // must be at least 2 less than kMaxTableBufSize
static int canvasSize = 300;
static int editWidth = 60;
static int editPosx = 1;
static int editPosy = 1;
static int editCursorPosx = 1;
static int editCursorPosy = 1;
static int squareWidth = 5;
static int running = 0;
static int userHideCursor = 0;
static unsigned int numGenerations;
#define EDIT_MODE 0
#define VIEW_MODE 1
static int displayModeToggle;

static char wasDeadRule[9] = {
                        0,
                        0,
                        0,
                        1,
                        0,
                        0,
                        0,
                        0,
                        0};

static char wasAliveRule[9] = {
                        0,
                        0,
                        1,
                        1,
                        0,
                        0,
                        0,
                        0,
                        0};

static void PlotGeneration(void);
static void PlotGenerationRedrawAll(int);
static void ComputeNextGeneration(void);
static void ChangeCellColor(void);
static void InitStartingConfiguration(void);
static void LoadIntoCurrentState(void);
static void SaveRingControlInfo(void);
static void ReadSettingsFromFile(void);
static void GetSettingsFromControls(void);
static void UpdateEditWindowPos(int xpos, int ypos);
static void UpdateEditCursorPos(int xpos, int ypos);


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    if ((panelHandle = LoadPanel (0, "life.uir", PANEL)) < 0)
        return -1;
    if ((popupPanel = LoadPanel (0, "life.uir", NAMEPOPUP)) < 0)
        return -1;
    if ((ringPanel = LoadPanel (panelHandle, "life.uir", RING_PANEL)) < 0)
        return -1;

    /* dynamically allocate memory for table */
    {
        register int i;
        
        /* Use calloc so that memory is initialized to zero
         * and thus the cleanup can be done more reliably if one
         * of the later mallocs fails.
         */
        table = (CellType**) calloc(kMaxTableBufSize, sizeof(CellType*));
        if(!table)
            goto OutOfMemory;
        for(i = 0; i < kMaxTableBufSize; i++) {
            table[i] = (CellType *) malloc(sizeof(CellType)*kMaxTableBufSize);
            if(!table[i])
                goto OutOfMemory;
        }
    }
 
    GetSettingsFromControls();
    GetProjectDir(projDir);
    ReadSettingsFromFile();
    InitStartingConfiguration();
    PlotGeneration();

    DisplayPanel (panelHandle);
    DisplayPanel (ringPanel);
    RunUserInterface ();

Done:    
    /* cleanup dynamically allocated memory for table */
    if(table) {
        register int i;
        
        for(i = 0; i < kMaxTableBufSize; i++)
            if(table[i])
                free(table[i]);
        free(table);
    }

    return 0;

OutOfMemory:
    MessagePopup("CVI Life", "Insufficent memory available to run.");
    goto Done;
}

static void PausePlayback(void)
{
    if(!running)
        return;
    running = 0;
    SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 0);
    SetCtrlAttribute (panelHandle, PANEL_PAUSE_BUTTON, ATTR_DIMMED, 1);
    PlotGenerationRedrawAll(running);
}

static void GetSettingsFromControls(void)
{
    double updateTime;
    // color
    ChangeCellColor();  
    // num generations
    GetCtrlVal (panelHandle, PANEL_GENERATIONS_NUMERIC, &numGenerations);
    // universe size
    GetCtrlVal (panelHandle, PANEL_SIZESLIDER, &tableSize);
    // mode
    GetCtrlVal (panelHandle, PANEL_EDITSWITCH, &displayModeToggle);
    // delay
    GetCtrlVal (panelHandle, PANEL_NUMERICSLIDE, &updateTime);
    SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_INTERVAL, updateTime);
    // edit win pos
    GetCtrlVal (panelHandle, PANEL_ORIGIN_X, &editPosx);
    GetCtrlVal (panelHandle, PANEL_ORIGIN_Y, &editPosy);
    UpdateEditWindowPos(editPosx, editPosy);
    // edit cursor pos
    GetCtrlVal (panelHandle, PANEL_CURSOR_X, &editCursorPosx);
    GetCtrlVal (panelHandle, PANEL_CURSOR_Y, &editCursorPosy);
    UpdateEditCursorPos(editCursorPosx, editCursorPosy);
    // hide cursor state
    GetCtrlVal (panelHandle, PANEL_HIDE_EDIT_TOGGLE, &userHideCursor);
}

static void ClearCurrentGen(void)
{
    int i,j;

    for(i=0; i<kMaxTableBufSize; i++)
        for(j=0; j<kMaxTableBufSize; j++) {
            table[i][j].stateChangedSinceLastDisplay = 0;
            table[i][j].isAlive = 0;
            table[i][j].nextGenNumNeighbors = 0;
            table[i][j].numNeighbors = 0;
        }
}

static void InitStartingConfiguration(void)
{
    ClearCurrentGen();
    LoadIntoCurrentState();
}

static void ChangeCellColor(void)
{
    int rgbval;
    
    GetCtrlVal (panelHandle, PANEL_COLORNUM, &rgbval);
    SetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_PEN_FILL_COLOR,
                      rgbval);
    SetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_PEN_COLOR,
                      rgbval);
}

static void ComputeNextGeneration(void)
{
    int i, j;
    
    for(i=1; i<=tableSize; i++)
        for(j=1; j<=tableSize; j++) {
            if(table[i][j].isAlive && !wasAliveRule[table[i][j].numNeighbors]) {
                // 1. update nextGenNumNeigbors of surrounding cells
                table[i-1][j-1].nextGenNumNeighbors--;
                table[i][j-1].nextGenNumNeighbors--;
                table[i+1][j-1].nextGenNumNeighbors--;
                table[i+1][j].nextGenNumNeighbors--;
                table[i+1][j+1].nextGenNumNeighbors--;
                table[i][j+1].nextGenNumNeighbors--;
                table[i-1][j+1].nextGenNumNeighbors--;
                table[i-1][j].nextGenNumNeighbors--;
                // 2. update numNeighbors of surrounding cells already processed
                table[i-1][j-1].numNeighbors--;
                table[i-1][j+1].numNeighbors--;
                table[i-1][j].numNeighbors--;
                table[i][j-1].numNeighbors--;
                // 3. change cells state to dead
                table[i][j].isAlive = 0;
                table[i][j].stateChangedSinceLastDisplay = 1;
            } else if(!table[i][j].isAlive && wasDeadRule[table[i][j].numNeighbors]) {
                // 1. update nextGenNumNeigbors of surrounding cells
                table[i-1][j-1].nextGenNumNeighbors++;
                table[i][j-1].nextGenNumNeighbors++;
                table[i+1][j-1].nextGenNumNeighbors++;
                table[i+1][j].nextGenNumNeighbors++;
                table[i+1][j+1].nextGenNumNeighbors++;
                table[i][j+1].nextGenNumNeighbors++;
                table[i-1][j+1].nextGenNumNeighbors++;
                table[i-1][j].nextGenNumNeighbors++;
                // 2. update numNeighbors of surrounding cells already processed
                table[i-1][j-1].numNeighbors++;
                table[i-1][j+1].numNeighbors++;
                table[i-1][j].numNeighbors++;
                table[i][j-1].numNeighbors++;
                // 3. change cells state to alive
                table[i][j].isAlive = 1;
                table[i][j].stateChangedSinceLastDisplay = 1;
            }
            // copy nextGenNumNeighbors to numNeighbors
            table[i][j].numNeighbors = table[i][j].nextGenNumNeighbors;
        }
}

static void PlotGeneration(void)
{
    int i, j;
    
    CanvasStartBatchDraw (panelHandle, PANEL_CANVAS);

    if(displayModeToggle == VIEW_MODE) {
        int centeringOffset = (canvasSize/2)-(tableSize/2);

        for(i=1; i<=tableSize; i++)
            for(j=1; j<=tableSize; j++) {
                if(table[i][j].stateChangedSinceLastDisplay) {
                    table[i][j].stateChangedSinceLastDisplay = 0;
                    if (table[i][j].isAlive) {
                        CanvasDrawPoint (panelHandle, PANEL_CANVAS,
                            MakePoint(i+centeringOffset-1, j+centeringOffset-1));
                    } else {
                        CanvasClear (panelHandle, PANEL_CANVAS,
                            MakeRect (j+centeringOffset-1, i+centeringOffset-1, 1, 1));
                    }
                }
            }
    } else {
        // edit mode redraw
        // draw cells
        for(i=editPosx; i<=editPosx+editWidth; i++)
            for(j=editPosy; j<=editPosy+editWidth; j++) {
                if(table[i][j].stateChangedSinceLastDisplay) {
                    table[i][j].stateChangedSinceLastDisplay = 0;
                    if (table[i][j].isAlive) {
                        CanvasDrawRect (panelHandle, PANEL_CANVAS,
                                        MakeRect((j-editPosy)*squareWidth,
                                         (i-editPosx)*squareWidth, squareWidth, squareWidth),
                                        VAL_DRAW_FRAME);
                    } else {
                        CanvasClear (panelHandle, PANEL_CANVAS,
                            MakeRect((j-editPosy)*squareWidth,
                             (i-editPosx)*squareWidth, squareWidth, squareWidth));
                    }
                }
            }
    }

    CanvasEndBatchDraw (panelHandle, PANEL_CANVAS);

}

static void PlotGenerationRedrawAll(int hideEditCursor)
{
    int i, j;
    
    CanvasStartBatchDraw (panelHandle, PANEL_CANVAS);
    CanvasClear (panelHandle, PANEL_CANVAS, VAL_ENTIRE_OBJECT);

    if(displayModeToggle == VIEW_MODE) {
        int centeringOffset = (canvasSize/2)-(tableSize/2);
        // draw a border box
        CanvasDrawRect (panelHandle, PANEL_CANVAS,
                        MakeRect(centeringOffset-1, centeringOffset-1, tableSize+2, tableSize+2),
                        VAL_DRAW_FRAME);

        // draw cells
        for(i=1; i<=tableSize; i++)
            for(j=1; j<=tableSize; j++) {
                if (table[i][j].isAlive) {
                    CanvasDrawPoint (panelHandle, PANEL_CANVAS,
                        MakePoint(i+centeringOffset-1, j+centeringOffset-1));
                }
            }

        if(!hideEditCursor && !userHideCursor) {
            // draw a box for the edit location
            SetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_PEN_COLOR,
                              0);
            CanvasDrawRect (panelHandle, PANEL_CANVAS,
                            MakeRect(centeringOffset+editPosy-1, centeringOffset+editPosx-1,
                                     editWidth, editWidth),
                            VAL_DRAW_FRAME);
            // set color back
            ChangeCellColor();
        }
    } else {
        // edit mode redraw
        // draw cells
        for(i=editPosx; i<=editPosx+editWidth; i++)
            for(j=editPosy; j<=editPosy+editWidth; j++) {
                if (table[i][j].isAlive) {
                    CanvasDrawRect (panelHandle, PANEL_CANVAS,
                                    MakeRect((j-editPosy)*squareWidth,
                                     (i-editPosx)*squareWidth, squareWidth, squareWidth),
                                    VAL_DRAW_FRAME);
                }
            }
        // draw edit cursor
        if(!hideEditCursor && !userHideCursor) {
            SetCtrlAttribute (panelHandle, PANEL_CANVAS, ATTR_PEN_COLOR,
                              0);
            CanvasDrawRect (panelHandle, PANEL_CANVAS,
                            MakeRect((editCursorPosy-editPosy)*squareWidth-squareWidth,
                             (editCursorPosx-editPosx)*squareWidth-squareWidth,
                             squareWidth+(squareWidth*2), squareWidth+(squareWidth*2)),
                            VAL_DRAW_FRAME);
            // set color back
            ChangeCellColor();
        }
    }

    CanvasEndBatchDraw (panelHandle, PANEL_CANVAS);

}

static void InsertCell(int xcoord, int ycoord)
{
    // update neighboring cells in array
    table[xcoord-1][ycoord-1].nextGenNumNeighbors++;
    table[xcoord-1][ycoord-1].numNeighbors++;
    table[xcoord][ycoord-1].nextGenNumNeighbors++;
    table[xcoord][ycoord-1].numNeighbors++;
    table[xcoord+1][ycoord-1].nextGenNumNeighbors++;
    table[xcoord+1][ycoord-1].numNeighbors++;
    table[xcoord+1][ycoord].nextGenNumNeighbors++;
    table[xcoord+1][ycoord].numNeighbors++;
    table[xcoord+1][ycoord+1].nextGenNumNeighbors++;
    table[xcoord+1][ycoord+1].numNeighbors++;
    table[xcoord][ycoord+1].nextGenNumNeighbors++;
    table[xcoord][ycoord+1].numNeighbors++;
    table[xcoord-1][ycoord+1].nextGenNumNeighbors++;
    table[xcoord-1][ycoord+1].numNeighbors++;
    table[xcoord-1][ycoord].nextGenNumNeighbors++;
    table[xcoord-1][ycoord].numNeighbors++;
    // mark cell as alive
    table[xcoord][ycoord].isAlive = 1;
    table[xcoord][ycoord].stateChangedSinceLastDisplay = 1; 
}

static void DeleteCell(int xcoord, int ycoord)
{
    // update neighboring cells in array
    table[xcoord-1][ycoord-1].nextGenNumNeighbors--;
    table[xcoord-1][ycoord-1].numNeighbors--;
    table[xcoord][ycoord-1].nextGenNumNeighbors--;
    table[xcoord][ycoord-1].numNeighbors--;
    table[xcoord+1][ycoord-1].nextGenNumNeighbors--;
    table[xcoord+1][ycoord-1].numNeighbors--;
    table[xcoord+1][ycoord].nextGenNumNeighbors--;
    table[xcoord+1][ycoord].numNeighbors--;
    table[xcoord+1][ycoord+1].nextGenNumNeighbors--;
    table[xcoord+1][ycoord+1].numNeighbors--;
    table[xcoord][ycoord+1].nextGenNumNeighbors--;
    table[xcoord][ycoord+1].numNeighbors--;
    table[xcoord-1][ycoord+1].nextGenNumNeighbors--;
    table[xcoord-1][ycoord+1].numNeighbors--;
    table[xcoord-1][ycoord].nextGenNumNeighbors--;
    table[xcoord-1][ycoord].numNeighbors--;
    // mark cell as dead
    table[xcoord][ycoord].isAlive = 0;
    table[xcoord][ycoord].stateChangedSinceLastDisplay = 1; 
}

static void ConvertCoordsToCellIndex(int *xcoord, int *ycoord)
{
    if(displayModeToggle == VIEW_MODE) {
        int centeringOffset = (canvasSize/2)-(tableSize/2);
        *xcoord = *xcoord+1-centeringOffset;
        *ycoord = *ycoord+1-centeringOffset;
    } else {
        *xcoord = (*xcoord / squareWidth)+1;
        *ycoord = (*ycoord / squareWidth)+1;
    }
}

int CVICALLBACK QuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 0);
            QuitUserInterface (0);
            break;
    }
    return 0;
}

int CVICALLBACK NextGenCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    unsigned int numGensToStep, i;
    switch (event) {
        case EVENT_COMMIT:
            PausePlayback();
            GetCtrlVal (panelHandle, PANEL_NUMTOSTEP, &numGensToStep);
            SetWaitCursor(1);
            for(i = 0; i < numGensToStep; i++) {
                numGenerations++;
                ComputeNextGeneration();
            }
            SetWaitCursor(0);
            SetCtrlVal (panelHandle, PANEL_GENERATIONS_NUMERIC, numGenerations);
            PlotGenerationRedrawAll(running);
            break;
    }
    return 0;
}

int CVICALLBACK TimerTick (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_TIMER_TICK:
            numGenerations++;
            SetCtrlVal (panelHandle, PANEL_GENERATIONS_NUMERIC, numGenerations);
            ComputeNextGeneration();
            PlotGeneration();
            break;
    }
    return 0;
}

int CVICALLBACK CanvasCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int xcoord, ycoord, buttonDown;
    int prevx = -1, prevy = -1;
    
    switch (event) {
        case EVENT_RIGHT_CLICK:
        case EVENT_RIGHT_DOUBLE_CLICK:
            GetRelativeMouseState (panelHandle, PANEL_CANVAS, &xcoord, &ycoord,
                                   NULL, &buttonDown, NULL);
            while(buttonDown){
                ConvertCoordsToCellIndex(&xcoord, &ycoord);
                if (displayModeToggle == EDIT_MODE) {
                    // add edit window offset to coords
                    xcoord += editPosx - 1;
                    ycoord += editPosy - 1;
                }
                
                if(prevx == xcoord && prevy == ycoord)
                    goto skip1;
                
                prevx = xcoord; prevy = ycoord;
                if(displayModeToggle == EDIT_MODE) {
                    // move edit window if mouse is off center
                    if(xcoord < (editPosx+(editWidth/2)))
                        editPosx--;
                    else if (xcoord > (editPosx+(editWidth/2)))
                        editPosx++;
                    if(ycoord < (editPosy+(editWidth/2)))
                        editPosy--;
                    else if (ycoord > (editPosy+(editWidth/2)))
                        editPosy++;
                } else {
                    // move edit window
                    // treat xcoord and ycoord as center of new location
                    editPosx = xcoord - editWidth/2;
                    editPosy = ycoord - editWidth/2;
                }
                // force coords within boundries
                if(editPosx < 1)
                    editPosx = 1;
                if(editPosy < 1)
                    editPosy = 1;
                if(editPosx > tableSize - editWidth + 1)
                    editPosx = tableSize - editWidth + 1;
                if(editPosy > tableSize - editWidth + 1)
                    editPosy = tableSize - editWidth + 1;
                // set new coords for editPos and redraw
                UpdateEditWindowPos(editPosx, editPosy);
                // If editCursorPos outside of editWindow
                // reset it as well.
                if(editCursorPosx < editPosx)
                    editCursorPosx = editPosx;
                if(editCursorPosy < editPosy)
                    editCursorPosy = editPosy;
                if(editCursorPosx > editPosx + editWidth-1)
                    editCursorPosx = editPosx + editWidth-1;
                if(editCursorPosy > editPosy + editWidth-1)
                    editCursorPosy = editPosy + editWidth-1;
                UpdateEditCursorPos(editCursorPosx, editCursorPosy);
                PlotGenerationRedrawAll(running);
skip1:
                GetRelativeMouseState (panelHandle, PANEL_CANVAS, &xcoord, &ycoord,
                                           NULL, &buttonDown, NULL);
            }
            break;
        case EVENT_LEFT_CLICK:
        case EVENT_LEFT_DOUBLE_CLICK:
            if(displayModeToggle != EDIT_MODE)
                break;
            GetRelativeMouseState (panelHandle, PANEL_CANVAS, &xcoord, &ycoord,
                                   &buttonDown, NULL, NULL);
            while(buttonDown){
                if(xcoord >= 0 && ycoord >=0 &&
                   xcoord < canvasSize && ycoord < canvasSize) { 
                    ConvertCoordsToCellIndex(&xcoord, &ycoord);
                    if (displayModeToggle == EDIT_MODE) {
                        // add edit window offset to coords
                        xcoord += editPosx - 1;
                        ycoord += editPosy - 1;
                    }

                    if(prevx == xcoord && prevy == ycoord)
                        goto skip2;

                    prevx = xcoord; prevy = ycoord;
                    // edit cells
                    if(!table[xcoord][ycoord].isAlive) {
                        InsertCell(xcoord, ycoord);
                    } else {
                        DeleteCell(xcoord, ycoord);
                    }
                    if(displayModeToggle == EDIT_MODE)
                        // move edit view cursor
                        UpdateEditCursorPos(xcoord, ycoord);
                    // redraw
                    PlotGenerationRedrawAll(running);
                }
skip2:
                GetRelativeMouseState (panelHandle, PANEL_CANVAS, &xcoord, &ycoord,
                                       &buttonDown, NULL, NULL);
            }
            break;
    }
    return 0;
}

int CVICALLBACK GoCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            running = 1;
            // force full redraw to clear cursor
            PlotGenerationRedrawAll(running);

            SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_ENABLED, 1);
            SetCtrlAttribute (panelHandle, PANEL_PAUSE_BUTTON, ATTR_DIMMED, 0);
            break;
    }
    return 0;
}

int CVICALLBACK PauseCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            PausePlayback();
            break;
    }
    return 0;
}

int CVICALLBACK SetTimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    double updateTime;
    
    switch (event) {
        case EVENT_VAL_CHANGED:
            GetCtrlVal (panelHandle, PANEL_NUMERICSLIDE, &updateTime);
            SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_INTERVAL, updateTime);
            break;
    }
    return 0;
}

int CVICALLBACK SetColorCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_VAL_CHANGED:
            ChangeCellColor();
            PlotGenerationRedrawAll(running);
            break;
    }
    return 0;
}

#define HELP_MSG                                                           \
    "                      A Simulation of Life\n\n"                       \
    "   Place cells anywhere on the canvas by holding down the left\n"     \
    "mouse button and dragging the mouse. Then either step one\n"          \
    "generation or push \"Go\" to step forward multiple generations.\n\n"  \
    "The rules of life are simple:\n"                                      \
    "    1) There are eight surrounding locations around any cell\n"       \
    "       or unoccupied location. (vertically, horizontally, and\n"      \
    "       diagonally)\n"                                                 \
    "    2) Any cell with 4 or more cells surrounding it will die of\n"    \
    "       overcrowding in the next generation.\n"                        \
    "    3) Any cell with fewer than 2 cells surrounding it will die\n"    \
    "       of loneliness in the next generation.\n"                       \
    "    4) Any empty location surrounded by exactly 3 cells will\n"       \
    "       contain a new cell in the next generation.\n\n"                \
    "Experiment: Certain shapes do certain things, some will shoot\n"      \
    "            across the screen, others will will generate such\n"      \
    "            shapes periodically. See what happens when shapes\n"      \
    "            collide. Enjoy!\n\n"                                      \
    "Note: Adjusting the universe size smaller will speed things up."


int CVICALLBACK HelpCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
                MessagePopup("CVI Life", HELP_MSG);
            break;
    }
    return 0;
}

int CVICALLBACK ResetCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            numGenerations = 0;
            SetCtrlVal (panelHandle, PANEL_GENERATIONS_NUMERIC, numGenerations);
            PausePlayback();
            ClearCurrentGen();
            PlotGenerationRedrawAll(running);
            break;
    }
    return 0;
}

static void SaveCurrentState(char *filename)
{
    FILE *pFile;
    int success = 0, i, j;
    int prevWaitCursor = GetWaitCursorState();
    
    pFile = fopen (filename, "wb");
    if(!pFile)
        goto Error;
    for(i=0; i<kMaxTableBufSize; i++)
        for(j=0; j<kMaxTableBufSize; j++)
            if(table[i][j].isAlive)
                saveTable[i][j] = 1;
            else
                saveTable[i][j] = 0;
    
    if(!fwrite (saveTable, sizeof(saveTable), 1, pFile))
        goto Error;
    
    success = 1;

Error:
    if(!success) {
        SetWaitCursor (0);
        MessagePopup("CVI Life","Unable to save current state. I/O Error.");
        SetWaitCursor(prevWaitCursor);
    }
    if(pFile)
        fclose(pFile);
    return;
}

static void LoadIntoCurrentState(void)
{
    FILE *pFile;
    int success = 0, i, j, itemIndex;
    char filename[MAX_FILENAME_LEN], pathname[MAX_PATHNAME_LEN];
    
    /* first make sure we are paused */
    PausePlayback();
    SetWaitCursor (1);

    // load previous panel info
    GetCtrlVal (ringPanel, RING_PANEL_SAVED_RING, &itemIndex);
    RecallPanelState (panelHandle, kLifeFilePnlStates, itemIndex);
    GetSettingsFromControls();

    // load life info
    sprintf(filename, kLifeFileFmtStr, itemIndex);
    MakePathname (projDir, filename, pathname);

    /* load file */
    pFile = fopen (pathname, "rb");
    if(!pFile)
        goto Error;
    if(!fread (saveTable, sizeof(saveTable), 1, pFile))
        goto Error;
        
    ClearCurrentGen();
    for(i=0; i<kMaxTableBufSize; i++)
        for(j=0; j<kMaxTableBufSize; j++)
            if(saveTable[i][j])
                InsertCell(i, j);
    
    success = 1;

Error:
    SetWaitCursor (0);
    if(!success)
        MessagePopup("CVI Life","Unable to load file. I/O Error or not a life file.");
    if(pFile)
        fclose(pFile);
    return;
}

int CVICALLBACK AddSavedCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            /* first make sure we are paused */
            PausePlayback();

            InstallPopup (popupPanel);
            break;
    }
    return 0;
}

int CVICALLBACK LoadCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            LoadIntoCurrentState();
            break;
    }
    return 0;
}

/* destFilename must be MAX_PATHNAME_LEN or greater */
static void FindNextAvailableLifeFile(int *currentItemIndex)
{
    int i, notUsed, foundOne = 0;
    char filename[MAX_PATHNAME_LEN];
    char pathname[MAX_PATHNAME_LEN];
    
    for(i=0; i<kMaxNumSavedFiles; i++) {
        // find a .lif file
        sprintf(filename, kLifeFileFmtStr, i);
        MakePathname (projDir, filename, pathname);
        if(GetFileSize (pathname, &notUsed) == -1) {
            /* we are successful */
            foundOne = 1;
            *currentItemIndex = i;
            break;
        }
    }
    
    if(!foundOne)
        *currentItemIndex = -1;
}

static void SaveRingControlInfo(void)
{
    char pathname[MAX_PATHNAME_LEN];
    char tmpBuf[MAX_PATHNAME_LEN];
    FILE * pFile;
    int numItems, i, success = 0, itemIndex;
    int prevWaitCursor = GetWaitCursorState();
    
    MakePathname (projDir, kSettingsFile, pathname);
    
    pFile = fopen (pathname, "wb");
    if(!pFile)
        goto Error;
        
    GetNumListItems (ringPanel, RING_PANEL_SAVED_RING, &numItems);
    
    /* write the number of items */
    if(!fwrite (&numItems, sizeof(int), 1, pFile))
        goto Error;
    
    for(i = 0; i < numItems; i++) {
        GetLabelFromIndex (ringPanel, RING_PANEL_SAVED_RING, i, tmpBuf);
        if(!fwrite (tmpBuf, strlen(tmpBuf), 1, pFile))
            goto Error;
        if(!fwrite ("\n", 1, 1, pFile))
            goto Error;
        GetValueFromIndex (ringPanel, RING_PANEL_SAVED_RING, i, &itemIndex);
        sprintf(tmpBuf, "%d", itemIndex);
        if(!fwrite (tmpBuf, strlen(tmpBuf), 1, pFile))
            goto Error;
        if(!fwrite ("\n", 1, 1, pFile))
            goto Error;
    }
    
    success = 1;

Error:
    if(!success) {
        SetWaitCursor(0);
        sprintf(tmpBuf, "Error writing settings file '%s'.", kSettingsFile);
        MessagePopup("CVI Life", tmpBuf);
        SetWaitCursor(prevWaitCursor);
    }
    if(pFile)
        fclose(pFile);
}

static void StripNewline(char buf[])
{
    int i;
    
    for(i = 0; i <= strlen(buf); i++)
        if(buf[i] == '\n')
            buf[i] = '\0';
}

static void ReadSettingsFromFile(void)
{
    char pathname[MAX_PATHNAME_LEN];
    char tmpBuf[MAX_PATHNAME_LEN];
    FILE * pFile;
    int numItems, i, success = 0;
    
    MakePathname (projDir, kSettingsFile, pathname);
    
    pFile = fopen (pathname, "rb");
    if(!pFile)
        goto Error;
        
    /* read in number of items */
    if(!fread (&numItems, sizeof(int), 1, pFile))
        goto Error;
        
    SetCtrlAttribute (ringPanel, RING_PANEL_SAVED_RING, ATTR_VISIBLE, 0);
    // clear list
    ClearListCtrl (ringPanel, RING_PANEL_SAVED_RING);

    for(i = 0; i < numItems; i++) {
        /* read in label */
        if(!fgets (tmpBuf, MAX_PATHNAME_LEN, pFile))
            goto Error;
        /* strip out '\n' char */
        StripNewline(tmpBuf);
        /* read in index */
        if(!fgets (pathname, MAX_PATHNAME_LEN, pFile))
            goto Error;
        /* strip out '\n' char */
        StripNewline(pathname);
        /* insert into list box */
        InsertListItem (ringPanel, RING_PANEL_SAVED_RING, -1, tmpBuf, atoi(pathname));
    }
    
    success = 1;

Error:
    SetCtrlAttribute (ringPanel, RING_PANEL_SAVED_RING, ATTR_VISIBLE, 1);
    if(!success) {
        sprintf(tmpBuf, "Error reading settings file '%s'.", kSettingsFile);
        MessagePopup("CVI Life", tmpBuf);
    }
    if(pFile)
        fclose(pFile);
}

int CVICALLBACK ContSaveCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char saveFilename[MAX_PATHNAME_LEN];
    char pathname[MAX_PATHNAME_LEN];
    char tmpBuf[MAX_PATHNAME_LEN];
    int currentItemIndex, success = 0;
    
    switch (event) {
        case EVENT_COMMIT:
            /* get name for label */
            GetCtrlVal (popupPanel, NAMEPOPUP_NAME_STRING, tmpBuf);
            if(strlen(tmpBuf) == 0) {
                MessagePopup("CVI Life", "You must supply a name.");
                break;
            }

            SetWaitCursor(1);

            FindNextAvailableLifeFile(&currentItemIndex);
            if (currentItemIndex == -1)
                goto Error;
            sprintf(saveFilename, kLifeFileFmtStr, currentItemIndex);
            MakePathname (projDir, saveFilename, pathname);
            /* add to listbox */
            InsertListItem (ringPanel, RING_PANEL_SAVED_RING, -1, tmpBuf, currentItemIndex);
            /* save data to file */
            SaveCurrentState(pathname);
            /* save current panel state */
            SavePanelState (panelHandle, kLifeFilePnlStates, currentItemIndex);
            /* save ring control settings */
            SaveRingControlInfo();
            RemovePopup (0);
            /* set string control back to empty string */
            SetCtrlVal (popupPanel, NAMEPOPUP_NAME_STRING, "");
            success = 1;
Error:
            SetWaitCursor(0);
            if (!success)
                MessagePopup("CVI Life", "Error: Unable to create save file.");
            break;
    }
    return 0;
}

int CVICALLBACK DelSavedCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char filename[MAX_PATHNAME_LEN];
    char pathname[MAX_PATHNAME_LEN];
    char labelName[MAX_PATHNAME_LEN];
    int itemIndex, panelIndex;
    
    switch (event) {
        case EVENT_COMMIT:
            /* first make sure we are paused */
            PausePlayback();

            GetCtrlIndex (ringPanel, RING_PANEL_SAVED_RING, &itemIndex);
            if(itemIndex < 0)
                break; /* list is empty */
            /* get label name for confirm popup */
            GetLabelFromIndex (ringPanel, RING_PANEL_SAVED_RING, itemIndex,
                               labelName);
            /* make sure the user is certain */
            sprintf(pathname, "Are you sure you wish to delete the item '%s'?"
                    , labelName); 
            if(ConfirmPopup ("CVI Life", pathname) == 0)
                break;
            GetCtrlVal (ringPanel, RING_PANEL_SAVED_RING, &panelIndex);
            sprintf(filename, kLifeFileFmtStr, panelIndex);
            MakePathname (projDir, filename, pathname);
            DeleteFile(pathname);
            DeleteListItem (ringPanel, RING_PANEL_SAVED_RING, itemIndex, 1);
            /* resave settings */
            SaveRingControlInfo();
            break;
    }
    return 0;
}

int CVICALLBACK EditChangeCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle, PANEL_EDITSWITCH, &displayModeToggle);
            PlotGenerationRedrawAll(running);
            break;
    }
    return 0;
}

int CVICALLBACK SizeChangeCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_VAL_CHANGED:
            GetCtrlVal (panelHandle, PANEL_SIZESLIDER, &tableSize);
            // make sure edit window is within new universe boundries
            UpdateEditWindowPos(editPosx, editPosy);
            PlotGenerationRedrawAll(running);
            break;
    }
    return 0;
}

static void UpdateEditWindowPos(int xpos, int ypos)
{
    editPosx = xpos;
    editPosy = ypos;
    
    // make sure edit window is inside universe boundry
    if(editPosx > tableSize - editWidth + 1)
        editPosx = tableSize - editWidth + 1;
    if(editPosy > tableSize - editWidth + 1)
        editPosy = tableSize - editWidth + 1;
    SetCtrlVal (panelHandle, PANEL_ORIGIN_X, editPosx);
    SetCtrlVal (panelHandle, PANEL_ORIGIN_Y, editPosy);
}

static void UpdateEditCursorPos(int xpos, int ypos)
{
    editCursorPosx = xpos;
    editCursorPosy = ypos;
    
    SetCtrlVal (panelHandle, PANEL_CURSOR_X, editCursorPosx);
    SetCtrlVal (panelHandle, PANEL_CURSOR_Y, editCursorPosy);
}

int CVICALLBACK PanelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    int virtualKey;
    switch (event) {
        case EVENT_KEYPRESS:
            if(running)
                break;
            virtualKey = eventData1 & VAL_VKEY_MASK;
/*          switch(virtualKey) {
                case VAL_UP_ARROW_VKEY:
                    if(editPosy == 1)
                        Beep();
                    else
                        UpdateEditWindowPos(editPosx, editPosy-1);
                    goto DidIt;
                case VAL_DOWN_ARROW_VKEY:
                    if(editPosy+1 > tableSize - editWidth + 1)
                        Beep();
                    else
                        UpdateEditWindowPos(editPosx, editPosy+1);
                    goto DidIt;
                case VAL_LEFT_ARROW_VKEY:
                    if(editPosx == 1)
                        Beep();
                    else
                        UpdateEditWindowPos(editPosx-1, editPosy);
                    goto DidIt;
                case VAL_RIGHT_ARROW_VKEY:
                    if(editPosx+1 > tableSize - editWidth + 1)
                        Beep();
                    else
                        UpdateEditWindowPos(editPosx+1, editPosy);
                    goto DidIt;
            }
*/          break;
    }
    return 0;

DidIt:
    PlotGenerationRedrawAll(running);
    return 1; // swallow keypress
}

int CVICALLBACK CancelSaveCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            RemovePopup (0);
            break;
    }
    return 0;
}

int CVICALLBACK HideCursorCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle, PANEL_HIDE_EDIT_TOGGLE, &userHideCursor);
            PlotGenerationRedrawAll(running);
            break;
    }
    return 0;
}

