/***************************************************************************

  Copyright 1996 National Instruments

****************************************************************************/

/*****************************************************************/
/***    Include files:                                         ***/
/*****************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include <utility.h>
#include "lissa.h"

/*****************************************************************/
/***    MACROS & Typedefs:                                     ***/
/*****************************************************************/

#define PI                                  3.141592654
#define TWO_PI                              6.283185307
#define NUM_PLOTS_PER_TIME_UPDATE           50
#define SETTINGS_LIST_PANEL_STATE_INDEX     -1
#define LISSA_SETTINGS_FILE                 "lissa.set"
#define Max(a,b)                            ((a > b) ? (a) : (b))

typedef double (CVIANSI *funcGenPtr) (double radians);

typedef struct
    {
    double      *waveForm;
    double      phase;
    double      radiansPerPoint;
    int         numPointsPerCycle;
    double      frequency;
    double      gain;
    funcGenPtr  waveTypeFuncPtr;
    }   ChannelWave;


/*****************************************************************/
/***    Function Prototypes                                    ***/
/*****************************************************************/

double  CVIANSI SawTooth(double radians);
double  CVIANSI Ramp(double radians);
double  CVIANSI CosTimesSin(double radians);
double  CVIANSI SineSeries(double radians);
double  CVIANSI SineExpansion(double radians);
double  CVIANSI SawTooth5Sine(double radians);
double  CVIANSI SawTooth10Sine(double radians);
double  CVIANSI Sine10Ramp(double radians);
void    InitChannel(ChannelWave * channel, double lengthOfTime, double timePerPoint);
void    InitChannelsFromUserSettings(void);
void    PlotChannels(void);
void    GetCurrentSettings(void);
void    SaveNewSetting(char *settingsName);
void    SaveOrLoadSettingsList(int save);
void    ApplySelectedSetting(void);
void    UpdateGraphBgColor(void);
void    UpdateTraceColor(void);
void    UpdateSpeedSetting(void);
void    UpdatePlotSpeedIndicator(void);
void    HandleFatalError(char *);
int     FindAlphabeticalInsertIndex(int panel, int listCtrl, char *newItemLabel);

/*****************************************************************/
/***    Variables:                                             ***/
/*****************************************************************/

ChannelWave gChannel1, gChannel2;
int         gPhaseAngle = 0, gPhaseDrift = 0, gPointsPerSweep;
int         gTraceColor, gBgColor;
int         gLissajousPanel;
double      gSweepPeriod;
int         gPanelSettingsListCtrl = PANEL_SAVEDSETTINGSLIST;
                /*  gPanelSettingsListCtrl must be a variable because the value
                    changes when the ctrl is deleted and recreated while
                    loading the settings */

funcGenPtr  waveFuncList[] =
    {
    sin,
    cos,
    tan,
    CosTimesSin,
    Ramp,
    SawTooth,
    SineSeries,
    SineExpansion,
    SawTooth5Sine,
    SawTooth10Sine,
    Sine10Ramp,
    0
    };


/*****************************************************************/
/***    Functions                                              ***/
/*****************************************************************/


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    gLissajousPanel = LoadPanel(0, "lissa.uir", PANEL);
    if (gLissajousPanel < 0)
        HandleFatalError("Failed to load panel from lissa.uir, aborting program.");

    SaveOrLoadSettingsList(0);
    ApplySelectedSetting();

    DisplayPanel(gLissajousPanel);

    RunUserInterface();

    return 0;
}

/*****************************************************************/

void CVICALLBACK ExitCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    QuitUserInterface(0);   /* user selected "Exit" from the menu */
}

/*****************************************************************/

void CVICALLBACK AboutLissaMenuItemCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    int aboutBox;
    
    aboutBox = LoadPanel(0, "lissa.uir", ABOUTBOX);

    InstallPopup(aboutBox);     /* display the about box */
}

/*****************************************************************/

int CVICALLBACK ExitAboutBoxCallback(int panel, int control, int event, void *callbackData,
                     int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        DiscardPanel(panel);    /* user clicked on "OK", remove the about box */

    return 0;
}

/*****************************************************************/

int CVICALLBACK PanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_CLOSE)   /* user selected "Close" from Windows system menu */
        QuitUserInterface(0);

    return 0;
}

/*****************************************************************/

int CVICALLBACK SaveSettingsCallback(int panel, int control, int event, void *callbackData,
                     int eventData1, int eventData2)
{
    char    settingsName[256];

    if (event == EVENT_COMMIT)
        {
            /* slow down plotting speed for better typing performance on slow machines */
         SetCtrlAttribute(gLissajousPanel, PANEL_TIMER, ATTR_INTERVAL, 0.075);

        if (GenericMessagePopup("Save Current Settings", "Type a name for these settings:",
                "__OK", "__Cancel", "", settingsName, 255, 0, VAL_GENERIC_POPUP_INPUT_STRING,
                VAL_GENERIC_POPUP_BTN1, VAL_GENERIC_POPUP_BTN2) == VAL_GENERIC_POPUP_BTN1)
            if (*settingsName)
                SaveNewSetting(settingsName);

        UpdateSpeedSetting();       /* restore plotting speed */
        }

    return 0;
}

/*****************************************************************/

int CVICALLBACK DeleteSettingCallback(int panel, int control, int event, void *callbackData,
                      int eventData1, int eventData2)
{
    int settingIndex, panelStateIndex;

    if (event == EVENT_COMMIT)
        {
        GetCtrlIndex(gLissajousPanel, gPanelSettingsListCtrl, &settingIndex);
        if (settingIndex >= 0) {
            DeleteListItem(gLissajousPanel, gPanelSettingsListCtrl, settingIndex, 1);
            SaveOrLoadSettingsList(1);
        }
    }

    return 0;
}

/*****************************************************************/

int CVICALLBACK ApplySettingCallback(int panel, int control, int event, void *callbackData,
                     int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        {
        ApplySelectedSetting();
        SetActiveCtrl(gLissajousPanel, gPanelSettingsListCtrl);
        }

    return 0;
}

/*****************************************************************/

/*
 * This function sets the lower frequency to be an integral fraction of the
 * larger frequency
 */
int CVICALLBACK AlignFrequenciesCallback(int panel, int control, int event, void *callbackData,
                         int eventData1, int eventData2)
{
    double  maxFreq, minFreq, freqRatio;
    double  *biggerFreq, *smallerFreq;

    if (event == EVENT_COMMIT)
        {
                /* assume max/min for both channels is the  same: */
        GetCtrlAttribute(panel, PANEL_CHAN1FREQUENCY, ATTR_MAX_VALUE, &maxFreq);
        GetCtrlAttribute(panel, PANEL_CHAN1FREQUENCY, ATTR_MIN_VALUE, &minFreq);
        maxFreq *= 1000;/* convert kHz to hz */
        minFreq *= 1000;/* convert kHz to hz */

        biggerFreq = (gChannel1.frequency > gChannel2.frequency) ? &gChannel1.frequency : &gChannel2.frequency;
        smallerFreq = (gChannel1.frequency < gChannel2.frequency) ? &gChannel1.frequency : &gChannel2.frequency;
        freqRatio = floor(*biggerFreq / *smallerFreq + .5);
        while (*biggerFreq / freqRatio < minFreq)
            freqRatio--;
        *smallerFreq = *biggerFreq / freqRatio;
        SetCtrlVal(panel, PANEL_CHAN1FREQUENCY, gChannel1.frequency / 1000);
        SetCtrlVal(panel, PANEL_CHAN2FREQUENCY, gChannel2.frequency / 1000);
        InitChannelsFromUserSettings(); /* update waveforms for new settings */
    }

    return 0;
}

/*****************************************************************/

int CVICALLBACK SettingsChangedCallback(int panel, int control, int event, void *callbackData,
                        int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED)
        InitChannelsFromUserSettings(); /* update waveforms for new settings */

    return 0;
}

/*****************************************************************/

int CVICALLBACK PhaseAngleCallback(int panel, int control, int event, void *callbackData,
                   int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED)
        GetCtrlVal(panel, control, &gPhaseAngle);

    return 0;
}

/*****************************************************************/

int CVICALLBACK PhaseDriftCallback(int panel, int control, int event, void *callbackData,
                   int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED)
        GetCtrlVal(panel, control, &gPhaseDrift);

    return 0;
}

/*****************************************************************/

void UpdateGraphBgColor(void)
{
    int color;  /*  a check is made to ensure the grid is always visible
                    regardless of the plot background color */

    GetCtrlVal(gLissajousPanel, PANEL_BGCOLOR, &color);
    if (color == VAL_BLACK)
        SetCtrlAttribute(gLissajousPanel, PANEL_SCOPEDISPLAY, ATTR_GRID_COLOR, VAL_WHITE);
    else
        SetCtrlAttribute(gLissajousPanel, PANEL_SCOPEDISPLAY, ATTR_GRID_COLOR, VAL_BLACK);

    SetCtrlAttribute(gLissajousPanel, PANEL_SCOPEDISPLAY, ATTR_PLOT_BGCOLOR, color);
}

/*****************************************************************/

int CVICALLBACK BgColorCallback(int panel, int control, int event, void *callbackData,
                int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED)
        UpdateGraphBgColor();

    return 0;
}

/*****************************************************************/

void UpdateTraceColor(void)
{
    GetCtrlVal(gLissajousPanel, PANEL_TRACECOLOR, &gTraceColor);
}

/*****************************************************************/

int CVICALLBACK TraceColorCallback(int panel, int control, int event, void *callbackData,
                   int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED)
        UpdateTraceColor();

    return 0;
}

/*****************************************************************/

void UpdateSpeedSetting(void)
{
    double timerInterval;

    GetCtrlVal(gLissajousPanel, PANEL_SPEED, &timerInterval);
    SetCtrlAttribute(gLissajousPanel, PANEL_TIMER, ATTR_INTERVAL, timerInterval);
}

/*****************************************************************/

int CVICALLBACK SpeedCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED)
        UpdateSpeedSetting();

    return 0;
}

/*****************************************************************/

int CVICALLBACK TimerCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_TIMER_TICK && gChannel1.waveForm && gChannel2.waveForm)
        {
        int plottingOff;

        GetCtrlVal(gLissajousPanel, PANEL_ONOFF, &plottingOff);
        if (!plottingOff)
            PlotChannels();

        UpdatePlotSpeedIndicator();
        }

    return 0;
}

/*****************************************************************/

void UpdatePlotSpeedIndicator(void)
{
    static double   lastTime = 0;
    static int      numPlotsSinceLastTimeStamp = 0;

    numPlotsSinceLastTimeStamp++;
    if (numPlotsSinceLastTimeStamp == NUM_PLOTS_PER_TIME_UPDATE)
        {
        double  newTime = Timer();
        double  elapsedTime = newTime - lastTime;

        numPlotsSinceLastTimeStamp = 0;
        lastTime = newTime;
        if (elapsedTime > 0)
            SetCtrlVal(gLissajousPanel, PANEL_PLOTSPERMINUTE, (int) (NUM_PLOTS_PER_TIME_UPDATE * 60 / elapsedTime));
    }
}

/*****************************************************************/

int FindAlphabeticalInsertIndex(int panel, int listCtrl, char *newItemLabel)
{
    char    currItemLabel[256];
    int     index, numItems;

    GetNumListItems(panel, listCtrl, &numItems);

    for (index = 0; index < numItems; index++)
        {
        GetLabelFromIndex(panel, listCtrl, index, currItemLabel);
        if (strcmp(newItemLabel, currItemLabel) < 0)
            return index;
        }

    return -1;      /* insert at end of list */
}

/*****************************************************************/

void SaveNewSetting(char *settingsName)
{
    int index, insertIndex, numItems, panelStateIndex = 0;

    do      /* find an unused panel state index to save current settings at */
        {
        panelStateIndex++;
        GetIndexFromValue(gLissajousPanel, gPanelSettingsListCtrl, &index, panelStateIndex);
        } while (index >= 0);

    insertIndex = FindAlphabeticalInsertIndex(gLissajousPanel, gPanelSettingsListCtrl, settingsName);
        /*  the panelStateIndex is stored as the value of the list box item */
    InsertListItem(gLissajousPanel, gPanelSettingsListCtrl, insertIndex, settingsName, panelStateIndex);

    GetNumListItems(gLissajousPanel, gPanelSettingsListCtrl, &numItems);
    SetCtrlIndex(gLissajousPanel, gPanelSettingsListCtrl, insertIndex == -1 ? numItems - 1 : insertIndex);
    DeleteGraphPlot(gLissajousPanel, PANEL_SCOPEDISPLAY, -1, 0);    /* don't waste space saving graph data */
    SavePanelState(gLissajousPanel, LISSA_SETTINGS_FILE, panelStateIndex);  /* save state of current settings */
    SaveOrLoadSettingsList(1);  /* save state of the list of settings */
}

/*****************************************************************/

/*
 * Creates a new panel to copy the settings list control to or from, and
 * recalls/saves the state of the panel to load/save the values in the
 * settings list control.  This special panel state is alway saved at panel
 * state index SETTINGS_LIST_PANEL_STATE_INDEX.
 */

void SaveOrLoadSettingsList(int save)
{
    int tempPanel, copyOfSettingsListCtrl, retval;

    tempPanel = NewPanel(0, "", 10, 10, 10, 10);
    copyOfSettingsListCtrl = DuplicateCtrl(gLissajousPanel, gPanelSettingsListCtrl, tempPanel,
             0, VAL_KEEP_SAME_POSITION, VAL_KEEP_SAME_POSITION);
    if (save)
        SavePanelState(tempPanel, LISSA_SETTINGS_FILE, SETTINGS_LIST_PANEL_STATE_INDEX);
    else
        {
        DisableBreakOnLibraryErrors();  /* don't break for fileNotFound,(the only reasonably likely error) */
        retval = RecallPanelState(tempPanel, LISSA_SETTINGS_FILE, SETTINGS_LIST_PANEL_STATE_INDEX);
        EnableBreakOnLibraryErrors();
        if (retval == -94)  /* -94 == fileNotFound */
            SaveOrLoadSettingsList(1);  /* create settings file if none exists */
        else
            {
            DiscardCtrl(gLissajousPanel, gPanelSettingsListCtrl);
            gPanelSettingsListCtrl = DuplicateCtrl(tempPanel, copyOfSettingsListCtrl, gLissajousPanel,
                            0, VAL_KEEP_SAME_POSITION, VAL_KEEP_SAME_POSITION);
            }
        }

    DiscardPanel(tempPanel);
}

/*****************************************************************/

void ApplySelectedSetting(void)
{
    int settingIndex, panelStateIndex;

        /* get index of currently selected setting */
    GetCtrlIndex(gLissajousPanel, gPanelSettingsListCtrl, &settingIndex);

    if (settingIndex >= 0)
        {
        GetValueFromIndex(gLissajousPanel, gPanelSettingsListCtrl, settingIndex, &panelStateIndex);
        RecallPanelState(gLissajousPanel, LISSA_SETTINGS_FILE, panelStateIndex);

            /*  restore the contents of the settings list box (it was changed
             *  along with the rest of the ctrls when the saved panel state was
             *  recalled)
             */
        SaveOrLoadSettingsList(0);

        SetCtrlIndex(gLissajousPanel, gPanelSettingsListCtrl, settingIndex);
        }

    InitChannelsFromUserSettings();
    UpdateGraphBgColor();
    UpdateTraceColor();
    UpdateSpeedSetting();
}

/*****************************************************************/

    /* this function does all the updating of the "scope" display */
void PlotChannels(void)
{
    int channel1Offset, channel2Offset;

    channel1Offset  = gChannel1.phase / gChannel1.radiansPerPoint;
    if (channel1Offset < 0)
        channel1Offset += gChannel1.numPointsPerCycle;
    gChannel1.phase += TWO_PI * gSweepPeriod * gChannel1.frequency + TWO_PI * gPhaseDrift / 360.0;
    gChannel1.phase = fmod(gChannel1.phase, TWO_PI);

    channel2Offset  = gChannel2.phase / gChannel2.radiansPerPoint;
    gChannel2.phase += +TWO_PI * gSweepPeriod * gChannel2.frequency;
    gChannel2.phase = fmod(gChannel2.phase, TWO_PI);

    gPhaseAngle = (gPhaseAngle + gPhaseDrift) % 360;    /* update phase drift */

    DeleteGraphPlot(gLissajousPanel, PANEL_SCOPEDISPLAY, -1, 0);
    PlotXY(gLissajousPanel, PANEL_SCOPEDISPLAY, gChannel1.waveForm + channel1Offset,
            gChannel2.waveForm + channel2Offset, gPointsPerSweep, VAL_DOUBLE,
            VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, gTraceColor);
}

/*****************************************************************/

double CVIANSI Ramp(double radians)
{
    return fmod(radians, TWO_PI) / PI - 1;
}

/*****************************************************************/

double CVIANSI CosTimesSin(double radians)
{
    return cos(radians) * sin(radians);
}

/*****************************************************************/

double CVIANSI SawTooth(double radians)
{
    double  val = 2 * fmod(radians, TWO_PI) / PI - 1;

    if (val > 1)
        val = 2 - val;  /* negative slope on second half of cycle */

    return val;
}

/*****************************************************************/

double CVIANSI SineSeries(double radians)
{
    return sin(radians) * sin(2 * radians) * sin(3 * radians) * sin(3 * radians) * 2;
}

/*****************************************************************/

double CVIANSI SineExpansion(double radians)
{
    return sin(radians) - sin(2 * radians) / 2 + sin(3 * radians) / 3 - sin(4 * radians) / 4 + sin(5 * radians) / 5;
}

/*****************************************************************/

double CVIANSI SawTooth5Sine(double radians)
{
    return SawTooth(5 * radians) * sin(radians);
}

/*****************************************************************/

double CVIANSI SawTooth10Sine(double radians)
{
    return SawTooth(10 * radians) * sin(radians);
}

/*****************************************************************/

double CVIANSI Sine10Ramp(double radians)
{
    return sin(radians * 10) * Ramp(radians);
}

/*****************************************************************/

/*
 * channel->gain, channel->waveTypeFuncPtr, and channel->frequency must be
 * initialized before InitChannel() is called
 */

void InitChannel(ChannelWave * channel, double lengthOfTime, double timePerPoint)
{
    double      *wavePtr;
    int         index;
    double      timePerCycle = 1 / channel->frequency;
    int         numPointsPerCycle = 1 / (channel->frequency * timePerPoint);
    int         numPoints = lengthOfTime / timePerPoint;
    double      radiansPerPoint = (TWO_PI * lengthOfTime / timePerCycle) / numPoints;
    double      currRadians, gain = channel->gain;
    funcGenPtr  waveFunc = channel->waveTypeFuncPtr;

    if (channel->waveForm)
        free(channel->waveForm);

    wavePtr = channel->waveForm = malloc(sizeof(double) * numPoints);
    if (!channel->waveForm)
        HandleFatalError("Not enough memory; exiting program...");

    channel->radiansPerPoint = radiansPerPoint;
    channel->numPointsPerCycle = numPointsPerCycle;

    for (index = currRadians = 0; index < numPoints; index++, currRadians += radiansPerPoint)
        *wavePtr++ = gain * (*waveFunc) (currRadians);
}

/*****************************************************************/

void InitChannelsFromUserSettings(void)
{
    double  timePerPoint, longestWavePeriod, totalWaveLength;

    GetCurrentSettings();

    longestWavePeriod = Max(1 / gChannel1.frequency, 1 / gChannel2.frequency);
    totalWaveLength = Max(longestWavePeriod, gSweepPeriod);
    timePerPoint = totalWaveLength / gPointsPerSweep;

    /*  Q:  Why multiply totalWaveLength by 2.1?
        A:  The 2 is so a full cycle available from any point in the first cycle.
            The 0.1 is to make sure any rounding errors don't leave us a point
            short (paranioa is a lifestyle
    */
    InitChannel(&gChannel1, totalWaveLength * 2.1, timePerPoint);
    InitChannel(&gChannel2, totalWaveLength * 2.1, timePerPoint);
}

/*****************************************************************/

void GetCurrentSettings(void)
{
    int waveTypeIndex;

    GetCtrlVal(gLissajousPanel, PANEL_PHASEDRIFT, &gPhaseDrift);
    GetCtrlVal(gLissajousPanel, PANEL_SAMPLESPERSWEEP, &gPointsPerSweep);
    GetCtrlVal(gLissajousPanel, PANEL_SWEEPPERIOD, &gSweepPeriod);
    gSweepPeriod /= 1000;   /* convert millisecs to seconds */

    GetCtrlVal(gLissajousPanel, PANEL_CHAN1FREQUENCY, &gChannel1.frequency);
    gChannel1.frequency *= 1000;    /* convert kHz to Hz */
    GetCtrlVal(gLissajousPanel, PANEL_CHAN1GAIN, &gChannel1.gain);
    GetCtrlVal(gLissajousPanel, PANEL_CHAN1WAVEFORMTYPE, &waveTypeIndex);
    gChannel1.waveTypeFuncPtr = waveFuncList[waveTypeIndex];

    GetCtrlVal(gLissajousPanel, PANEL_CHAN2FREQUENCY, &gChannel2.frequency);
    gChannel2.frequency *= 1000;    /* convert kHz to Hz */
    GetCtrlVal(gLissajousPanel, PANEL_CHAN2GAIN, &gChannel2.gain);
    GetCtrlVal(gLissajousPanel, PANEL_CHAN2WAVEFORMTYPE, &waveTypeIndex);
    gChannel2.waveTypeFuncPtr = waveFuncList[waveTypeIndex];

    GetCtrlVal(gLissajousPanel, PANEL_TRACECOLOR, &gTraceColor);
    GetCtrlVal(gLissajousPanel, PANEL_BGCOLOR, &gBgColor);
}

/*****************************************************************/

void HandleFatalError(char *message)
{
    MessagePopup("Error", message);
    exit(0);
}

/**********************************************/
