/*****************************************************************************/
/* This program demonstrates the use of intensity plots by graphing the      */
/* mandelbrot fractal images.  By changing the position of either the graph  */ 
/* cursors or the X & Y range controls, you can zoom in or out on the        */
/* current graph image.  You can also change the interpolation options to    */
/* see the plotting differences                                              */
/*                                                                           */
/* PlotIntensity options:                                                    */
/*                                                                           */
/* Interpolate Colors - This controls the Interp Colors parameter of         */
/*                  PlotIntensity(). When enbaled, it allows that function   */
/*                  to perform the interpolation described above. In this    */
/*                  program, the array passed is only a minimal 2 entry      */
/*                  array consisting of the Low color/Low value entry and    */
/*                  the High Color/High value entry. The Hi Color parameter  */
/*                  in this case is not meaningful, since the array itself   */
/*                  contains the full range of the data.                     */
/*                                                                           */
/* Interpolate Pixels -     This controls the Interp Pixels parameter of     */
/*                  PlotIntensity(). When enabled, every pixel within the    */
/*                  rectangular area of the plot will be interpolated from   */
/*                  the values of the 4 points which map to the nearest      */
/*                  pixels. In this case, the plotting time is determined    */
/*                  not by the size of the data array, but by the size in    */
/*                  pixels of the plot, which since this sample uses an      */
/*                  autoscaled graph, corresponds to the full size of the    */
/*                  plot area of the graph.                                  */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************/
/***    Include files:                                         ***/
/*****************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <ansi_c.h>
#include <userint.h>
#include <utility.h>         
#include "mandel.h"

/*****************************************************************/
/***    MACROS & Typedefs:                                     ***/
/*****************************************************************/

#define MAXSIZE   10
#define RES 150
#define MANDEL_SETTINGS_FILE  "mandel.set" 
#define SETTINGS_LIST_PANEL_STATE_INDEX  -1

#define Min(a,b) ((a)<(b))?(a):(b)
#define Max(a,b) ((a)>(b))?(a):(b)


typedef struct {
    double      xmin;
    double      xmax;
    double      ymin;
    double      ymax;
    unsigned short maxit;
    int intcolor;
    int intpixel;
}   MandelSet;
    
/*****************************************************************/
/***    Global Variables:                                      ***/
/*****************************************************************/

ColorMapEntry colors[16];
float zarray[RES][RES];
int undo_flag=0;

MandelSet mset;
int gMandelPanel;
MandelSet gOldSettings;

/*  gPanelSettingsList; must be a variable because the value
    changes when the ctrl is deleted and recreated while
    loading the settings 
*/
int gPanelSettingsList = PNL_SAVEDSETTINGSLIST;

/*****************************************************************/
/***    Function Declarations:                                 ***/
/*****************************************************************/

void PlotMandel(MandelSet);
double MapCur(double min,double max,double cur);
void SaveNewSetting(char *settingsName);
void SaveOrLoadSettingsList(int save);
void ApplySelectedSetting(void);
int FindAlphabeticalInsertIndex(int panel, int listCtrl, char *newItemLabel);



/*****************************************************************/ 

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    gMandelPanel = LoadPanel (0, "mandel.uir", PNL);
    
    SaveOrLoadSettingsList(0);  
    
    SetGraphCursor (gMandelPanel, PNL_GRAPH, 1, RES*0.05, RES*0.05);
    SetGraphCursor (gMandelPanel, PNL_GRAPH, 2, RES*0.95, RES*0.95);
    
    mset.xmin=-2.0; mset.xmax= 1.0;
    mset.ymin=-1.5; mset.ymax= 1.5;
    
    GetCtrlVal (gMandelPanel, PNL_MAXIT, &mset.maxit);
    GetCtrlVal (gMandelPanel, PNL_INTCOLOR, &mset.intcolor);
    GetCtrlVal (gMandelPanel, PNL_INTPIXEL, &mset.intpixel);
    
    DisplayPanel (gMandelPanel);
    
    PlotMandel(mset);
        
    RunUserInterface ();

    return 0;
}

/*****************************************************************/

int CVICALLBACK plotcb (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{

  double xcur1, ycur1, xcur2, ycur2;

  switch (event) {
   case EVENT_COMMIT:
   
    gOldSettings=mset;
   
    GetCtrlVal (gMandelPanel, PNL_MAXIT, &mset.maxit);
    GetCtrlVal (gMandelPanel, PNL_INTCOLOR, &mset.intcolor);
    GetCtrlVal (gMandelPanel, PNL_INTPIXEL, &mset.intpixel);
    
    GetCtrlVal (gMandelPanel, PNL_XMIN, &mset.xmin);
    GetCtrlVal (gMandelPanel, PNL_XMAX, &mset.xmax);
    GetCtrlVal (gMandelPanel, PNL_YMIN, &mset.ymin);
    GetCtrlVal (gMandelPanel, PNL_YMAX, &mset.ymax);
    
    PlotMandel(mset);
    
    SetGraphCursor (gMandelPanel, PNL_GRAPH, 1, RES*0.05, RES*0.05);
    SetGraphCursor (gMandelPanel, PNL_GRAPH, 2, RES*0.95, RES*0.95);
    
    break;
  }
  return 0;
}

/*****************************************************************/

int CVICALLBACK quitcb (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
    QuitUserInterface(0);
            break;
    }
    return 0;
}

/*****************************************************************/

int CVICALLBACK RangeChange (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    double x1,y1,x2,y2;
    double xmin,ymin,xmax,ymax;
    double scx1,scy1,scx2,scy2;
 
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (gMandelPanel, PNL_XMIN, &xmin);
            GetCtrlVal (gMandelPanel, PNL_XMAX, &xmax);
            GetCtrlVal (gMandelPanel, PNL_YMIN, &ymin);
            GetCtrlVal (gMandelPanel, PNL_YMAX, &ymax);
            
            break;
        
    }
    return 0;
}

/*****************************************************************/


int CVICALLBACK MoveCursor (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    double x1,y1,x2,y2;
    double scx1,scy1,scx2,scy2;
 
    switch (event) {
        case EVENT_VAL_CHANGED:
            GetGraphCursor (gMandelPanel, PNL_GRAPH, 1, &x1, &y1);
            GetGraphCursor (gMandelPanel, PNL_GRAPH, 2, &x2, &y2);
            
            scx1= MapCur(mset.xmin,mset.xmax,x1);
            scx2= MapCur(mset.xmin,mset.xmax,x2);
            scy1= MapCur(mset.ymin,mset.ymax,y1);
            scy2= MapCur(mset.ymin,mset.ymax,y2);
            
            SetCtrlVal (gMandelPanel, PNL_CURX1, scx1);
            SetCtrlVal (gMandelPanel, PNL_CURX2, scx2); 
            SetCtrlVal (gMandelPanel, PNL_CURY1, scy1); 
            SetCtrlVal (gMandelPanel, PNL_CURY2, scy2);
            
            SetCtrlVal (gMandelPanel, PNL_XMIN, Min(scx1, scx2));
            SetCtrlVal (gMandelPanel, PNL_XMAX, Max(scx1, scx2)); 
            SetCtrlVal (gMandelPanel, PNL_YMIN, Min(scy1, scy2)); 
            SetCtrlVal (gMandelPanel, PNL_YMAX, Max(scy1, scy2));
            
            break;
        
    }
    return 0;
}

/*****************************************************************/

double MapCur(double min,double max,double cur)
{
    double dif;

    dif = max - min; 
    return (cur / RES * (dif) + min);
}

/*****************************************************************/

void PlotMandel(MandelSet mset)
{
    int i, j;
    double x, y, zreal, zimag, real, imag;
    int mandel, exit;
    double minvalue, maxvalue, value;
    double xInc, yInc;
    
    SetMouseCursor (VAL_HOUR_GLASS_CURSOR);
    ResetTextBox (gMandelPanel, PNL_STATUS, "GENERATING IMAGE");
    ProcessDrawEvents ();

    minvalue = mset.maxit;
    maxvalue = 0;

    xInc = (mset.xmax - mset.xmin) / RES;
    yInc = (mset.ymax - mset.ymin) / RES;
    
    y = mset.ymin;
    for (i = 0; i < RES; i++) 
    {
        x = mset.xmin;
        for (j = 0; j < RES; j++) 
        {
            /* calculate mandelbrot value */
            mandel = 0;
            exit = 0;
            zreal = 0.0;
            zimag = 0.0;                          
            
            while ((mandel <= mset.maxit) && !exit) 
            {
                real = (zreal * zreal) - (zimag * zimag) + x;
                imag = (2.0 * zreal * zimag) + y;
                if (((real * real) + (imag * imag)) >= MAXSIZE) 
                    exit = 1;
                mandel++;
                zreal = real;
                zimag = imag;
            
            }
            
            value = (double)(mandel - 1);
            zarray[i][j] = value;
            if (value > maxvalue) 
                maxvalue = value;
            else if (value < minvalue) 
                minvalue = value;
    
            x += xInc;
        }
        y += yInc;
    }
    
    /* Define 16 colors to represent the intensity values */
    colors[0].color = VAL_WHITE;
    colors[1].color = VAL_RED;         
    colors[2].color = VAL_BLUE;        
    colors[3].color = VAL_GREEN;       
    colors[4].color = VAL_CYAN;        
    colors[5].color = VAL_MAGENTA;     
    colors[6].color = VAL_YELLOW;      
    colors[7].color = VAL_DK_RED;      
    colors[8].color = VAL_DK_BLUE;     
    colors[9].color = VAL_DK_GREEN;    
    colors[10].color = VAL_DK_CYAN;     
    colors[11].color = VAL_DK_MAGENTA; 
    colors[12].color = VAL_DK_YELLOW;  
    colors[13].color = VAL_LT_GRAY;    
    colors[14].color = VAL_DK_GRAY;    
    colors[15].color = VAL_BLACK;      
    
    /* Define numerical values for the intensity colors */
    value = minvalue;   
    for (i=0; i<16; i++) {
        colors[i].dataValue.valFloat = (float)(value);
        value = value + ((maxvalue - minvalue) / 15.0);
    }    
    
    ResetTextBox(gMandelPanel, PNL_STATUS, "PLOTTING IMAGE");
    ProcessDrawEvents ();
    
    DeleteGraphPlot (gMandelPanel, PNL_GRAPH, -1, VAL_DELAYED_DRAW);
    
    PlotIntensity (gMandelPanel, PNL_GRAPH, zarray, RES,RES, VAL_FLOAT,
            colors, VAL_BLACK ,16, mset.intcolor, mset.intpixel);

    if(undo_flag)
        SetCtrlAttribute (gMandelPanel, PNL_UNDO, ATTR_DIMMED, 0); 
    else
        undo_flag=1;

    ResetTextBox(gMandelPanel, PNL_STATUS, "Push PLOT After\n  Providing Input");
        
    SetCtrlVal (gMandelPanel, PNL_CURX1, mset.xmin);
    SetCtrlVal (gMandelPanel, PNL_CURX2, mset.xmax);
    SetCtrlVal (gMandelPanel, PNL_CURY1, mset.ymin);
    SetCtrlVal (gMandelPanel, PNL_CURY2, mset.ymax);
    
    SetMouseCursor (VAL_DEFAULT_CURSOR);

}

/*****************************************************************/

int CVICALLBACK SaveSettingsCallback(int panel, int control, int event, void *callbackData,
                     int eventData1, int eventData2)
{
    char    settingsName[256];

    if (event == EVENT_COMMIT) {
        
        if (GenericMessagePopup("Save Current Settings", "Type a name for these settings:",
                "__OK", "__Cancel", "", settingsName, 255, 0, VAL_GENERIC_POPUP_INPUT_STRING,
                VAL_GENERIC_POPUP_BTN1, VAL_GENERIC_POPUP_BTN2) == VAL_GENERIC_POPUP_BTN1)
            if (*settingsName)
                SaveNewSetting(settingsName);

        }

    return 0;
}

/*****************************************************************/

int CVICALLBACK DeleteSettingCallback(int panel, int control, int event, void *callbackData,
                      int eventData1, int eventData2)
{
    int settingIndex, panelStateIndex;

    if (event == EVENT_COMMIT) {
        GetCtrlIndex(gMandelPanel, gPanelSettingsList, &settingIndex);
        if (settingIndex >= 0) {
            DeleteListItem(gMandelPanel, gPanelSettingsList, settingIndex, 1);
            SaveOrLoadSettingsList(1);
        }
    }

    return 0;
}

/*****************************************************************/

int CVICALLBACK ApplySettingCallback(int panel, int control, int event, void *callbackData,
                     int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        ApplySelectedSetting();
        SetActiveCtrl(gMandelPanel, gPanelSettingsList);
        }

    return 0;
}


/*****************************************************************/

int FindAlphabeticalInsertIndex(int panel, int listCtrl, char *newItemLabel)
{
    char    currItemLabel[256];
    int     index, numItems;

    GetNumListItems(panel, listCtrl, &numItems);

    for (index = 0; index < numItems; index++) {
        GetLabelFromIndex(panel, listCtrl, index, currItemLabel);
        if (strcmp(newItemLabel, currItemLabel) < 0)
            return index;
    }

    return -1;      /* insert at end of list */
}

/*****************************************************************/

void SaveNewSetting(char *settingsName)
{
    int index, insertIndex, numItems, panelStateIndex = 0;

    do  {    /* find an unused panel state index to save current settings at */
        panelStateIndex++;
        GetIndexFromValue(gMandelPanel, gPanelSettingsList, &index, panelStateIndex);
    } while (index >= 0);

    insertIndex = FindAlphabeticalInsertIndex(gMandelPanel, gPanelSettingsList, settingsName);
        /*  the panelStateIndex is stored as the value of the list box item */
    InsertListItem(gMandelPanel, gPanelSettingsList, insertIndex, settingsName, panelStateIndex);

    GetNumListItems(gMandelPanel, gPanelSettingsList, &numItems);
    SetCtrlIndex(gMandelPanel, gPanelSettingsList, insertIndex == -1 ? numItems - 1 : insertIndex);
    
    DeleteGraphPlot (gMandelPanel, PNL_GRAPH, -1, VAL_NO_DRAW);
    
    SavePanelState(gMandelPanel, MANDEL_SETTINGS_FILE, panelStateIndex);  /* save state of current settings */
    SaveOrLoadSettingsList(1);  /* save state of the list of settings */
}

/*****************************************************************/

/*
   Creates a new panel to copy the settings list control to or from, and
   recalls/saves the state of the panel to load/save the values in the
   settings list control.  This special panel state is alway saved at panel
   state index SETTINGS_LIST_PANEL_STATE_INDEX.
*/

void SaveOrLoadSettingsList(int save)
{
    int tempPanel, copyOfSettingsListCtrl, retval;

    tempPanel = NewPanel(0, "", 10, 10, 10, 10);
    copyOfSettingsListCtrl = DuplicateCtrl(gMandelPanel, gPanelSettingsList, tempPanel,
             0, VAL_KEEP_SAME_POSITION, VAL_KEEP_SAME_POSITION);
    if (save)
        SavePanelState(tempPanel, MANDEL_SETTINGS_FILE, SETTINGS_LIST_PANEL_STATE_INDEX);
    else {
        DisableBreakOnLibraryErrors();  /* don't break for fileNotFound,(the only reasonably likely error) */
        retval = RecallPanelState(tempPanel, MANDEL_SETTINGS_FILE, SETTINGS_LIST_PANEL_STATE_INDEX);
        EnableBreakOnLibraryErrors();
        if (retval == -94)  /* -94 == fileNotFound */
            SaveOrLoadSettingsList(1);  /* create settings file if none exists */
        else {
            DiscardCtrl(gMandelPanel, gPanelSettingsList);
            gPanelSettingsList = DuplicateCtrl(tempPanel, copyOfSettingsListCtrl, gMandelPanel,
                            0, VAL_KEEP_SAME_POSITION, VAL_KEEP_SAME_POSITION);
        }
    }

    DiscardPanel(tempPanel);
}

/*****************************************************************/

void ApplySelectedSetting(void)
{
    int settingIndex, panelStateIndex;
    
     gOldSettings=mset;

        /* get index of currently selected setting */
    GetCtrlIndex(gMandelPanel, gPanelSettingsList, &settingIndex);

    if (settingIndex >= 0) {
        GetValueFromIndex(gMandelPanel, gPanelSettingsList, settingIndex, &panelStateIndex);
        RecallPanelState(gMandelPanel, MANDEL_SETTINGS_FILE, panelStateIndex);

        /*  restore the contents of the settings list box (it was changed
            along with the rest of the ctrls when the saved panel state was
            recalled) 
        */
        SaveOrLoadSettingsList(0);

        SetCtrlIndex(gMandelPanel, gPanelSettingsList, settingIndex);
    }
        
    GetCtrlVal (gMandelPanel, PNL_XMIN, &mset.xmin);
    GetCtrlVal (gMandelPanel, PNL_XMAX, &mset.xmax); 
    GetCtrlVal (gMandelPanel, PNL_YMIN, &mset.ymin); 
    GetCtrlVal (gMandelPanel, PNL_YMAX, &mset.ymax); 
    GetCtrlVal (gMandelPanel, PNL_MAXIT, &mset.maxit);
    GetCtrlVal (gMandelPanel, PNL_INTCOLOR, &mset.intcolor);
    GetCtrlVal (gMandelPanel, PNL_INTPIXEL, &mset.intpixel);
    PlotMandel(mset);
    SetGraphCursor (gMandelPanel, PNL_GRAPH, 1, RES*0.05, RES*0.05);
    SetGraphCursor (gMandelPanel, PNL_GRAPH, 2, RES*0.95, RES*0.95);
}

/*****************************************************************/   

int CVICALLBACK InterpPixels_cb (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char message[255];
    switch (event) {
        case EVENT_RIGHT_CLICK:
            sprintf(message,"%s%s%s","Turning on this control causes the intensity ",
                "graph to interpolate data.\nThis will effectively smooth the ",
                "image in the graph, but takes longer to plot." );
            MessagePopup ("Interpolate Pixels Help",message);

            break;
    }
    return 0;
}

/*****************************************************************/   

int CVICALLBACK Interpcolor_cb (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char message[255]; 
    switch (event) {
        case EVENT_RIGHT_CLICK:
            sprintf(message,"%s%s%s","Turning on this control causes the intensity ",
                "graph to interpolate color values.\nThis means that CVI will fill ",
                "holes left in the user defined color table.");
            MessagePopup ("Interpolate Colors Help",message);
            break;
    }
    return 0;
}

/*****************************************************************/

int CVICALLBACK undocb (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            PlotMandel(gOldSettings);
            SetGraphCursor (gMandelPanel, PNL_GRAPH, 1, RES*0.05, RES*0.05);
            SetGraphCursor (gMandelPanel, PNL_GRAPH, 2, RES*0.95, RES*0.95);
            SetCtrlVal (gMandelPanel, PNL_XMIN, gOldSettings.xmin);
            SetCtrlVal (gMandelPanel, PNL_YMIN, gOldSettings.ymin);
            SetCtrlVal (gMandelPanel, PNL_XMAX, gOldSettings.xmax);
            SetCtrlVal (gMandelPanel, PNL_YMAX, gOldSettings.ymax);
            SetCtrlVal (gMandelPanel, PNL_INTPIXEL, gOldSettings.intpixel);
            SetCtrlVal (gMandelPanel, PNL_INTCOLOR, gOldSettings.intcolor);
            SetCtrlVal (gMandelPanel, PNL_MAXIT, gOldSettings.maxit);
            SetCtrlVal (gMandelPanel, PNL_CURX1, gOldSettings.xmin);
            SetCtrlVal (gMandelPanel, PNL_CURY1, gOldSettings.ymin);
            SetCtrlVal (gMandelPanel, PNL_CURX2, gOldSettings.xmax);
            SetCtrlVal (gMandelPanel, PNL_CURY2, gOldSettings.ymax);
            gOldSettings=mset;
            break;
    }
    return 0;
}
