/*****************************************************************/
/*  Filter Analysis Source Code                                  */
/*                                                               */
/*       This program acquires an analog input waveform from a   */
/*  DAQ board and plots it to the screen.  A data window may be  */
/*  selected and applied to the sampled points prior to          */
/*  calculation of the FFT and power spectrum of the signal.     */
/*  The power spectrum of the input signal is computed using the */
/*  formula   magnitude (FFT(X)/(n^2)), where n is the number of */
/*  points in the input sampled waveform.  The AutoPowerSpectrum */
/*  command in the Advanced Analysis Library performs the        */
/*  calculation.  The user can view either the windowed and      */
/*  filtered (if enabled) time waveform or the frequency         */
/*  spectrum by selecting the appropriate function on the switch */
/*  on the main panel.  The board, channel, sampling rate, and   */
/*  number of points analyzed per iteration may be set by        */
/*  clicking on the DAQ Configuration button.  Filtering of the  */
/*  windowed input signal may be enabled by clicking on the      */
/*  Filters button.  The signal plotted on the lower graph will  */
/*  reflect the effects of the selected filter provided that the */
/*  filtering enable switch has been turned on.  Please note     */
/*  that the cutoff frequency values must be less than one-half  */
/*  of the sampling frequency to avoid aliasing.  Also, for      */
/*  bandpass and bandstop filters, the upper cutoff frequency    */
/*  must be set to a value greater than the lower cutoff         */
/*  frequency.  An error message will warn the user if an        */
/*  invalid number is entered.  Help is available for each       */
/*  control by right-clicking on the control.  General help      */
/*  information is repeated on-line by clicking on the Help      */
/*  button.                                                      */
/*                                                               */
/*  Source Code Description:                                     */
/*  A brief description is provided below for each callback      */
/*  function.  A timer on the main panel calls the Getdata       */
/*  function as rapidly as possible.  This function is the heart */
/*  of the program and acquires the input waveform, windows it,  */
/*  calculates the power spectrum, filters the waveform if       */
/*  desired, and plots the input waveform and either the power   */
/*  spectrum or time waveform of the windowed and filtered       */
/*  signal to the upper and lower graphs, respectively.  Other   */
/*  callback functions generally acquire data values from        */
/*  data input controls.  More information is provided below.    */
/*                                                               */
/*****************************************************************/

#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include "spectrum.h" 
#include <ansi_c.h>
#include <analysis.h>
#include <dataacq.h>
#include <userint.h>
#include <utility.h> 
#include <easyio.h>

/*****************************************************************/
static int helppnl;
static int displaymode = 0;
static double samprate = 20000.0;
static int points = 512;
static char channel[50] = "1";
static int board = 1;
static filtercode = 17;
static int running = 0;
static int mainpnl;
static int filterpnl;
static int scalepnl, daqpanel;
static int i;
static int scale;
static int datawindowtype;
static double rate;
static unsigned long task;
static unsigned long backlog;
static double waveform[2048];
static WindowConst constants;
static double freqspace;
static double spectrum[(2048 + 79)];
static double datawinplot[2048];
static int filterenable = 0;
static int filttype = 0;
static int iirtype = 0;
static int iirorder = 5;
static double iircutoff1 = 5000;
static double iircutoff2 = 8000;
static int filterfunc = 0;
static double passbndrip = .50;
static double stpbndatt = 40;
static int firfiltype = 0;
static int firwindow = 1;
static int firlength = 49;
static double fircutoff1 = 5000;
static double fircutoff2 = 8000;
static double kaiserbeta = 1.00;
static double filtcoefs[80];
static double filteredwave[(2048 + 79)];
static initial = 1;
static int daqpnl;
static int graphscalepnl;  
static double waveformymin = -10.0;
static double waveformymax = 10.0;
static double spectrumlinmin = 0.0;
static double spectrumlinmax = 5.0;
static double spectrumdbmin = -200.0;
static double spectrumdbmax = 30.0;
static double spectrumdbmmin = -140.0;
static double spectrumdbmmax = 90.0;
static double timewavemin = -20.0;
static double timewavemax = 20.0;
static int signalswitch = 0;
static int simsignaltype = 0;
static double simsignalfreq = 2000.0;
static double simphase;
static short int errorchk = 0;
static short int boardcode;
static short int status = 0;

void myfoo(void);

/*****************************************************************/
/*                                                               */
/*  Variable Descriptions:                                       */
/*                                                               */
/*  1.  helppnl - help panel handle                              */
/*  2.  displaymode - selects between frequency spectrum and     */
/*                    time waveform displays for the lower graph.*/
/*                    0 = frequency spectrum (default)           */
/*                    1 = time waveform                          */
/*  3.  samprate - sampling rate in Hertz (default = 20000 Hz)   */
/*  4.  points - number of points to be acquired and processed   */
/*               with each call to the Getdata function          */
/*               (default = 512)                                 */ 
/*  5.  channel - string specifying the channel on the DAQ board */
/*                to acquire data from  (default = "1")          */
/*  6.  board - board to acquire data from (default = 1)         */
/*  7.  filtercode - a numeric code specifying which type of     */
/*                   filter has been chosen by the user.  The    */
/*                   filter codes are numbered 1 through 24 and  */
/*                   are listed below (default = 17)             */
/*                                                               */
/*      FILTER CODES                                             */
/*      1 = LP BUTTERWORTH IIR                                   */
/*      2 = HP BUTTERWORTH IIR                                   */
/*      3 = BP BUTTERWORTH IIR                                   */
/*      4 = BS BUTTERWORTH IIR                                   */
/*      5 = LP CHEBYSHEV IIR                                     */
/*      6 = HP CHEBYSHEV IIR                                     */
/*      7 = BP CHEBYSHEV IIR                                     */
/*      8 = BS CHEBYSHEV IIR                                     */
/*      9 = LP INVERSE CHEBYSHEV IIR                             */
/*     10 = HP INVERSE CHEBYSHEV IIR                             */
/*     11 = BP INVERSE CHEBYSHEV IIR                             */
/*     12 = BS INVERSE CHEBYSHEV IIR                             */
/*     13 = LP ELLIPTIC IIR                                      */
/*     14 = HP ELLIPTIC IIR                                      */
/*     15 = BP ELLIPTIC IIR                                      */
/*     16 = BS ELLIPTIC IIR                                      */
/*     17 = LP WINDOW FIR                                        */
/*     18 = HP WINDOW FIR                                        */
/*     19 = BP WINDOW FIR                                        */
/*     20 = BS WINDOW FIR                                        */
/*     21 = LP KAISER FIR                                        */
/*     22 = HP KAISER FIR                                        */
/*     23 = BP KAISER FIR                                        */
/*     24 = BS KAISER FIR                                        */
/*                                                               */
/*  8.  running - indicator variable which is a 0 if data        */
/*                acquisition is disabled and a 1 if enabled     */
/*                (default = 0)                                  */
/*  9.  mainpnl - main panel handle                              */
/* 10.  filterpnl - filter selection panel handle                */
/* 11.  i - count variable used in for loops                     */
/* 12.  scale - variable for switching between linear, dB, and   */
/*              dBm scales for the vertical axis on the power    */
/*              spectrum graph.                                  */
/*              0 = linear scale (default)                       */
/*              1 = dB scale                                     */
/*              2 = dBm scale                                    */
/* 13.  datawindowtype - number specifying which type of window  */
/*                       to be applied to the data.              */
/*                       0 = Uniform (default)                   */
/*                       1 = Hanning                             */
/*                       2 = Hamming                             */
/*                       3 = Blackman-Harris                     */
/*                       4 = Exact Blackman                      */
/*                       5 = Blackman                            */
/*                       6 = Flat Top                            */
/*                       7 = 4 Term Blackman-Harris              */
/*                       8 = 7 Term Blackman-Harris              */
/* 14.  rate - variable used with the easyio data acquisition    */
/*             functions                                         */
/* 15.  task - variable used with the easyio data acquisition    */
/*             functions                                         */
/* 16.  backlog - variable used with the easyio data acquisition */
/*                functions                                      */
/* 17.  waveform - buffer holding the acquired waveform points   */
/*                 both before and after windowing               */
/* 18.  constants - holds various parameters resulting from the  */
/*                  ScaledWindow function                        */
/* 19.  freqspace - frequency spacing returned by the            */
/*                  AutoPowerSpectrum function                   */
/* 20.  spectrum - holds the power spectrum points both before   */
/*                 and after filtering                           */
/* 21.  datawinplot - array holding the points used in plotting  */
/*                    the general shape of the selected data     */
/*                    window.  These points are not actually     */
/*                    used in the windowing process.             */
/* 22.  filterenable - variable indicating if filtering has been */
/*                     enabled.                                  */
/*                     0 = filtering disabled (default)          */
/*                     1 = filtering enabled                     */
/* 23.  filttype - variable indicating which class of filters    */
/*                 has been selected.                            */
/*                 0 = FIR filters  (default)                    */
/*                 1 = IIR filters                               */
/* 24.  iirtype - variable indicating which type of iir filter   */
/*                prototype has been selected.                   */
/*                0 = Butterworth (default)                      */
/*                1 = Chebyshev                                  */
/*                2 = Inverse Chebyshev                          */
/*                3 = Elliptic                                   */
/* 25.  iirorder - order of the iir filter (default = 5)         */
/* 26.  iircutoff1 - cutoff frequency for lowpass and highpass   */
/*                   IIR filters.  Lower cutoff frequency for    */
/*                   bandpass and bandstop IIR filters.          */
/*                   (default = 5000 Hz)                         */
/* 27.  iircutoff2 - upper cutoff frequency for bandpass and     */
/*                   bandstop IIR filters                        */
/*                   (default = 8000 Hz)                         */
/* 28.  filterfunc - indicates which type of filter has been     */
/*                   selected                                    */
/*                   0 = lowpass (default)                       */
/*                   1 = highpass                                */
/*                   2 = bandpass                                */
/*                   3 = bandstop                                */
/* 29.  passbndrip - ripples in the passband (in dB) of          */
/*                   Chebyshev and Elliptic filters              */
/*                   (default = .50 dB)                          */
/* 30.  stpbndatt - attenuation in the stopband (in dB) of       */
/*                  Inverse Chebyshev and Elliptic filters       */
/*                  (default = 40 dB)                            */
/* 31.  firfiltype - indicates which type of FIR filter has been */
/*                   selected                                    */
/*                   0 = Window method (default)                 */
/*                   1 = Kaiser window method                    */
/* 32.  firwindow - indicates which window has been chosen for   */
/*                  the FIR Window method of filter design       */
/*                  1 = Rectangular (default)                    */
/*                  2 = Triangular                               */
/*                  3 = Hanning                                  */
/*                  4 = Hamming                                  */
/*                  5 = Blackman                                 */
/* 33.  firlength - length of the FIR filter (default = 50)      */
/* 34.  fircutoff1 - cutoff frequency for lowpass and highpass   */
/*                   FIR filters; lower cutoff frequency for     */
/*                   bandpass and bandstop FIR filters           */
/*                   (default = 5000 Hz)                         */
/* 35.  fircutoff2 - upper cutoff frequency for bandpass and     */
/*                   bandstop FIR filters (default = 8000 Hz)    */
/* 36.  kaiserbeta - value of the beta parameter used in the     */
/*                   Kaiser window design method (default = 1.0) */
/* 37.  filtcoefs - array that holds the FIR filter coefficients */
/*                  generated by the FIR filter commands         */
/* 38.  filteredwave - array that holds the points of the input  */
/*                     waveform after filtering by the IIR or    */
/*                     FIR filtering commands                    */
/* 39.  initial - indicates if the filtering menu is being       */
/*                entered for the first time after running the   */
/*                program.  Used only for internal interface     */
/*                managment purposes.                            */
/*                1 = system in initial state (default)          */
/*                0 = filter menu has been altered from initial  */
/*                    state                                      */
/* 40.  daqpnl - data acquisition configuration panel handle     */
/* 41.  graphscalepnl - graph scaling panel handle               */
/* 42.  waveformymin - minimum value for y axis on upper graph   */
/* 43.  waveformymax - maximum value for y axis on upper graph   */
/* 44.  spectrumlinmin - minimum value for y axis on lower graph */
/*                       for linear scale mode                   */
/* 45.  spectrumlinmax - maximum value for y axis on lower graph */
/*                       for linear scale mode                   */
/* 46.  spectrumdbmin - minimum value for y axis on lower graph  */
/*                      for dB scale mode                        */
/* 47.  spectrumdbmax - maximum value for y axis on lower graph  */
/*                      for dB scale mode                        */
/* 48.  spectrumdbmmin - minimum value for y axis on lower graph */
/*                       for dBm scale mode                      */
/* 49.  spectrumdbmmax - maximum value for y axis on lower graph */
/*                       for dBm scale mode                      */
/* 50.  timewavemin - minimum value for y axis on lower graph    */
/*                    for time waveform display mode             */
/* 51.  timewavemax - maximum value for y axis on lower graph    */
/*                    for time waveform display mode             */
/*                                                               */
/*****************************************************************/

/*****************************************************************/
/*                                                               */
/* Main function description:                                    */
/*                                                               */
/* The main function initializes datawinplot to a uniform window */
/* and then plots this window in the window graph in the top     */
/* left of the main panel.  The five panels are loaded, the main */
/* panel is displayed, and the RunUserInterface command ends the */
/* function.                                                     */
/*                                                               */
/*****************************************************************/

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    if ((graphscalepnl = LoadPanel (0, "spectrum.uir", SCALEPNL)) < 0)
        return -1;
    if ((helppnl = LoadPanel (0, "spectrum.uir", HELPPNL)) < 0)
        return -1;
    if ((daqpnl = LoadPanel (0, "spectrum.uir", DAQPANEL)) < 0)
        return -1;
    if ((filterpnl = LoadPanel (0, "spectrum.uir", FILTERPNL)) < 0)
        return -1;
    if ((mainpnl = LoadPanel (0, "spectrum.uir", MAINPNL)) < 0)
        return -1;
    
    for (i=0; i<points; i++) 
    {
        datawinplot[i] =1;
    }
    
    PlotY (mainpnl, MAINPNL_WINDOWGRAPH, datawinplot, points, VAL_DOUBLE,
           VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);

    SetInputMode (daqpnl, DAQPANEL_SIGNALSWITCH, 1);
    
    DisplayPanel (mainpnl);
    RunUserInterface ();
    
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/* Getdatawindow Function Description:                           */
/*                                                               */
/* The Getdatawindow function is called whenever the user        */
/* changes the data window used.  The function obtains the new   */
/* value of the data window from the control and places it into  */
/* the datawindowtype variable.  The data window is calculated   */
/* and placed into the datawinplot arraw.  The old window plot   */
/* is erased and the new window is plotted.                      */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getdatawindow (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (mainpnl, MAINPNL_DATAWINDOW, &datawindowtype);
            switch (datawindowtype) 
            {
                case 0:
                    for (i=0; i<points; i++) 
                        datawinplot[i] = 1;
                    break;
                case 1:
                    for (i=0; i<points; i++) 
                        datawinplot[i] = .5 - (.5*cos(2*3.14*i/points));
                    break;
                case 2:
                    for (i=0; i<points; i++) 
                        datawinplot[i] = .54 - (.46*cos(2*3.14*i/points));
                    break;
                case 3:
                    for (i=0; i<points; i++) 
                        datawinplot[i] = .42323 - (.49755*cos(2*3.14*i/points)) + (.07922*cos(4*3.14*i/points));
                    break;
                case 4:
                    for (i=0; i<points; i++) 
                        datawinplot[i] = (7938.0/18608.0)-((9240.0/18608.0)*cos(2*3.14*i/points))+((1430.0/18608.0)*cos(4*3.14*i/points));
                    break;
                case 5:
                    for (i=0; i<points; i++) 
                        datawinplot[i] = .42-(.5*cos(2*3.14*i/points))+(.08*cos(4*3.14*i/points));
                    break;
                case 6:
                    for (i=0; i<points; i++) 
                        datawinplot[i] = .2810639-(.5208972*cos(2*3.14*i/points))+(.1980399*cos(4*3.14*i/points));
                    break;
                case 7:
                    for (i=0; i<points; i++) 
                        datawinplot[i] = .42323-(.49755*cos(2*3.14*i/points))+(.07922*cos(4*3.14*i/points));
                    break;
                case 8:
                    for (i=0; i<points; i++) 
                        datawinplot[i] = .42323-(.49755*cos(2*3.14*i/points))+(.07922*cos(4*3.14*i/points));
                    break;
            }
            DeleteGraphPlot (mainpnl, MAINPNL_WINDOWGRAPH, -1, VAL_DELAYED_DRAW);
            PlotY (mainpnl, MAINPNL_WINDOWGRAPH, datawinplot, points, VAL_DOUBLE,
                   VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);

            break;
    case EVENT_RIGHT_CLICK:
       MessagePopup ("Data Window Selection",
                     "Selects the window to be applied to the acquired signal.");
       break;
    }

    
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/* Getfilter Function Description:                               */
/*                                                               */
/* The Getfilter function is called whenever the user left-      */
/* clicks on the Filters button.  The function hides the main    */
/* panel, displays the filter selection panel, and disables the  */
/* timer.  Thus, no data is acquired while the filter selection  */
/* panel is being displayed.  If the user right-clicks on the    */
/* Filters button, this function is called and a help message is */
/* displayed containing information about the control.           */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getfilter (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            DisplayPanel (filterpnl);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Filter Parameters",
                          "Displays and allows changes to filter parameters.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Start Function Description:                                  */
/*                                                               */
/*  The Start function is called when the user clicks on the     */
/*  Start button.  If the user left-clicks on the button, data   */
/*  acquisition is configured with easyio DAQ functions.  The    */
/*  timer is enabled, which allows the Getdata function to be    */
/*  called for processing the data.  The running flag is set to  */
/*  1 to indicate that data acquisition has been enabled.  If    */
/*  user right-clicks on the Start button, a help message is     */
/*  displayed containing information about the control.          */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Start (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            if (signalswitch == 1) {
            
               AIClearAcquisition (task);
               AIStartAcquisition (board, channel, 2*points, samprate, 10, 0, &rate, &task);
            }
            SetCtrlAttribute (mainpnl, MAINPNL_TIMER, ATTR_ENABLED, 1);
            SetInputMode(mainpnl, MAINPNL_START, 0);  
            SetInputMode(mainpnl, MAINPNL_STOP,  1);  
            running = 1;
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Start Data Acquisition",
                          "Begins signal acquisition and processing.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Stop Function Description:                                   */
/*                                                               */
/*  The Stop function is called when the user clicks on the Stop */
/*  button.  When the user left-clicks on the Stop button, the   */
/*  timer that calls the Getdata function is disabled, thereby   */
/*  stopping all data acquisition and signal processing.  The    */
/*  running flag is set to zero to indicate that data            */
/*  processing has stopped.  If the user right-clicks on the     */
/*  Stop button, this function displays a help message with      */
/*  information about the Stop button control.                   */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Stop (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            SetCtrlAttribute (mainpnl, MAINPNL_TIMER, ATTR_ENABLED, 0);
            SetInputMode(mainpnl, MAINPNL_START, 1);  
            SetInputMode(mainpnl, MAINPNL_STOP,  0);  
            running = 0;
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Stop Data Acquisition",
                          "Disables signal acquisition and processing.");
            break; 
    }       
    
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/* Quit Function Description:                                    */
/*                                                               */
/* The Quit function is called whenever the user clicks on the   */
/* Quit button.  If the user left-clicks on the Quit button,     */
/* this function quits the program using the QuitUserInterface   */
/* command.  If the user right-clicks on the Quit button, the    */
/* function displays a help message with information about the   */
/* Quit button control.                                          */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Quit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Quit",
                          "Quits the program.");
            break;            
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getscale Function Description:                               */
/*                                                               */
/*  The Getscale function is called whenever the user changes    */
/*  setting on the scale switch.  If the user left-clicks on the */
/*  control, the function reads the new value of the control and */
/*  puts the value in the variable scale.  If the user right-    */
/*  clicks on the control, a help message is displayed with      */
/*  information about the scale control.                         */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getscale (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (mainpnl, MAINPNL_SCALE, &scale);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Axis",
                          "Sets the axis on the frequency graph to linear, dB, or dBm.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getdata Function Description:                                */
/*                                                               */
/*  The Getdata function is called with every clock tick from    */
/*  the Timer on the main panel.  The timer must be enabled by   */
/*  clicking on the Start DAQ button before this function can be */
/*  called.  The function reads data from the data acquisition   */
/*  buffer and puts it into the waveform variable.  The old time */
/*  waveform plot on the upper graph is deleted and the new time */
/*  waveform of the acquired data is plotted.  The data is then  */
/*  scaled with the selected data window using the ScaledWindow  */
/*  command.  If filtering is disabled, the power spectrum is    */
/*  computed with the AutoPowerSpectrum command and is placed    */
/*  the spectrum array.  If the graph display switch is set to   */
/*  frequency spectrum, the previous plot on the lower graph     */
/*  will be deleted and the current frequency spectrum will be   */
/*  plotted using the selected vertical scale.  If the graph     */
/*  display switch is set to time waveform, the windowed time    */
/*  waveform will be plotted to show the effects of the data     */
/*  window.  If filtering has been enabled, the switch on the    */
/*  filtercode variable will select the appropriate filtering    */
/*  procedure.  The windowed waveform will be filtered and       */
/*  placed into the filteredwave array.  The Convolve command is */
/*  necessary for FIR filters since the filter commands only     */
/*  generate the filter coefficients, which are placed into the  */
/*  filtercoefs array.  The power spectrum of the filtered wave  */
/*  is computed and the appropriate graph is plotted according   */
/*  to switch settings.                                          */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getdata (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_TIMER_TICK:
            if (signalswitch == 0) {
                switch (simsignaltype) {
                    case 0:
                        SineWave (points, 5.0, (simsignalfreq/20000.0), &simphase, waveform);
                        
                        break;
                    case 1:
                        SquareWave (points, 5.0, (simsignalfreq/20000.0), &simphase, 50.0,
                                    waveform);
                        
                        break;
                    case 2:
                        TriangleWave (points, 5.0, (simsignalfreq/20000.0), &simphase,
                                      waveform);
                        
                        break;
                    case 3:
                        Impulse (points, 100.0, 0, waveform);
                        
                        break;
                }
            }    
            else {
               status = AIReadAcquisition (task, points, LATEST_MODE, &backlog,GROUP_BY_CHANNEL, waveform);
               if (abs(status) == 10846) {
                   SetCtrlAttribute (mainpnl, MAINPNL_TIMER, ATTR_ENABLED, 0);
                   MessagePopup ("DAQ Error",
                                 "The sampling rate you have selected is too large for your computer.  Data in the buffer is being overwritten.  Please change your sampling rate to a lower value.");
                   DisplayPanel (daqpnl);
                  
               }
            }
            
            DeleteGraphPlot (mainpnl, MAINPNL_TIMEGRAPH, -1, VAL_DELAYED_DRAW);
            SetAxisRange (mainpnl, MAINPNL_TIMEGRAPH, VAL_NO_CHANGE, 0.0, 1.0,
                          VAL_MANUAL, waveformymin, waveformymax);
            
            PlotY (mainpnl, MAINPNL_TIMEGRAPH, waveform, points, VAL_DOUBLE,
                   VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
            ProcessDrawEvents ();
            
            ScaledWindow (waveform, points, datawindowtype, &constants);
            if (filterenable == 0) {
                AutoPowerSpectrum (waveform, points, (1/samprate), spectrum,
                                   &freqspace);
                DeleteGraphPlot (mainpnl, MAINPNL_FREQGRAPH, -1, VAL_DELAYED_DRAW);
                if (displaymode==0) {
                    
                switch (scale) {
                    case 0:
                        SetAxisRange (mainpnl, MAINPNL_FREQGRAPH, VAL_NO_CHANGE, 0.0, 1.0,
                                      (simsignaltype==3)?VAL_AUTOSCALE:VAL_MANUAL, spectrumlinmin, spectrumlinmax);
                        
                        break;
                    case 1:
                        for (i=0; i<(points/2); i++) {
                            spectrum[i] = 20*(log10(spectrum[i]));
                        }
                        SetAxisRange (mainpnl, MAINPNL_FREQGRAPH, VAL_NO_CHANGE, 0.0, 1.0,
                                      (simsignaltype==3)?VAL_AUTOSCALE:VAL_MANUAL, spectrumdbmin, spectrumdbmax);
                        
                        break;
                    case 2:
                        for (i=0; i<(points/2); i++) {
                            spectrum[i] = 20*(log10(spectrum[i]/.001));
                        }
                        SetAxisRange (mainpnl, MAINPNL_FREQGRAPH, VAL_NO_CHANGE, 0.0, 1.0,
                                      (simsignaltype==3)?VAL_AUTOSCALE:VAL_MANUAL, spectrumdbmmin, spectrumdbmmax);
                        
                        break;
                }
                
                PlotWaveform (mainpnl, MAINPNL_FREQGRAPH, spectrum, points/2,
                              VAL_DOUBLE, 1.0, 0.0, 0.0, freqspace, VAL_THIN_LINE,
                              VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
                }
                else {
                   
                   SetAxisRange (mainpnl, MAINPNL_FREQGRAPH, VAL_AUTOSCALE, 0.0, 1.0,
                                 VAL_MANUAL, timewavemin, timewavemax);
                   
                   PlotY (mainpnl, MAINPNL_FREQGRAPH, waveform, points, VAL_DOUBLE,
                   VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);     
                }
            }
            else {
                switch (filtercode) {
                    case 1:  /* LP BUTTERWORTH IIR */
                        Bw_LPF (waveform, points, samprate, iircutoff1, iirorder,
                                filteredwave);
                        break;
                    case 2:  /* HP BUTTERWORTH IIR */
                        Bw_HPF (waveform, points, samprate, iircutoff1, iirorder,
                                filteredwave);
                        break;
                    case 3:  /* BP BUTTERWORTH IIR */
                        Bw_BPF (waveform, points, samprate, iircutoff1, iircutoff2, iirorder,
                                filteredwave);
                        break;
                    case 4:  /* BS BUTTERWORTH IIR */
                        Bw_BSF (waveform, points, samprate, iircutoff1, iircutoff2, iirorder,
                                filteredwave);
                        break;
                    case 5:  /* LP CHEBYSHEV IIR */
                        Ch_LPF (waveform, points, samprate, iircutoff1, passbndrip, iirorder,
                                filteredwave);
                        break;
                    case 6:  /* HP CHEBYSHEV IIR */
                        Ch_HPF (waveform, points, samprate, iircutoff1, passbndrip, iirorder,
                                filteredwave);
                        break;
                    case 7:  /* BP CHEBYSHEV IIR */
                        Ch_BPF (waveform, points, samprate, iircutoff1, iircutoff2,
                                passbndrip, iirorder, filteredwave);
                        break;
                    case 8:  /* BS CHEBYSHEV IIR */
                        Ch_BSF (waveform, points, samprate, iircutoff1, iircutoff2,
                                passbndrip, iirorder, filteredwave);
                        break;
                    case 9:  /* LP INVERSE CHEBYSHEV IIR */
                        InvCh_LPF (waveform, points, samprate, iircutoff1, stpbndatt,
                                   iirorder, filteredwave);
                        break;
                    case 10: /* HP INVERSE CHEBYSHEV IIR */
                        InvCh_HPF (waveform, points, samprate, iircutoff1, stpbndatt,
                                   iirorder, filteredwave);
                        break;
                    case 11: /* BP INVERSE CHEBYSHEV IIR */
                        InvCh_BPF (waveform, points, samprate, iircutoff1, iircutoff2,
                                   stpbndatt, iirorder, filteredwave);
                        break;
                    case 12: /* BS INVERSE CHEBYSHEV IIR */
                        InvCh_BSF (waveform, points, samprate, iircutoff1, iircutoff2,
                                   stpbndatt, iirorder, filteredwave);
                        break;
                    case 13: /* LP ELLIPTIC IIR */
                        Elp_LPF (waveform, points, samprate, iircutoff1, passbndrip,
                                 stpbndatt, iirorder, filteredwave);
                        break;
                    case 14: /* HP ELLIPTIC IIR */
                        Elp_HPF (waveform, points, samprate, iircutoff1, passbndrip,
                                 stpbndatt, iirorder, filteredwave);
                        break;
                    case 15: /* BP ELLIPTIC IIR */
                        Elp_BPF (waveform, points, samprate, iircutoff1, iircutoff2,
                                 passbndrip, stpbndatt, iirorder, filteredwave);
                        break;
                    case 16: /* BS ELLIPTIC IIR */
                        Elp_BSF (waveform, points, samprate, iircutoff1, iircutoff2,
                                 passbndrip, stpbndatt, iirorder, filteredwave);
                        break;
                    case 17: /* LP WINDOW FIR */
                        Wind_LPF (samprate, fircutoff1, firlength, filtcoefs, firwindow);
                        Convolve (filtcoefs, firlength, waveform, points, filteredwave);
                        break;
                    case 18: /* HP WINDOW FIR */
                        Wind_HPF (samprate, fircutoff1, firlength, filtcoefs, firwindow);
                        Convolve (filtcoefs, firlength, waveform, points, filteredwave); 
                        break;
                    case 19: /* BP WINDOW FIR */
                        Wind_BPF (samprate, fircutoff1, fircutoff2, firlength, filtcoefs, firwindow);
                        Convolve (filtcoefs, firlength, waveform, points, filteredwave); 
                        break;
                    case 20: /* BS WINDOW FIR */
                        Wind_BSF (samprate, fircutoff1, fircutoff2, firlength, filtcoefs,firwindow);
                        Convolve (filtcoefs, firlength,waveform,points, filteredwave); 
                        break;
                    case 21: /* LP KAISER FIR */
                        Ksr_LPF (samprate, fircutoff1, firlength, filtcoefs, kaiserbeta);
                        Convolve (filtcoefs, firlength, waveform, points, filteredwave); 
                        break;
                    case 22: /* HP KAISER FIR */
                        Ksr_HPF (samprate, fircutoff1, firlength, filtcoefs, kaiserbeta);
                        Convolve (filtcoefs, firlength, waveform, points, filteredwave); 
                        break;
                    case 23: /* BP KAISER FIR */
                        Ksr_BPF (samprate, fircutoff1, fircutoff2, firlength, filtcoefs, kaiserbeta);
                        Convolve (filtcoefs, firlength, waveform, points, filteredwave); 
                        break;
                    case 24: /* BS KAISER FIR */
                        Ksr_BSF (samprate, fircutoff1, fircutoff2, firlength, filtcoefs, kaiserbeta);
                        Convolve (filtcoefs,firlength, waveform, points, filteredwave); 
                        break;
                }  /* END OF SWITCH */
                AutoPowerSpectrum (filteredwave, points, (1/samprate), spectrum,
                                   &freqspace);
                DeleteGraphPlot (mainpnl, MAINPNL_FREQGRAPH, -1, VAL_DELAYED_DRAW);
                if (displaymode == 0) {
                
                    switch (scale) {
                        case 0:
                            SetAxisRange (mainpnl, MAINPNL_FREQGRAPH, VAL_NO_CHANGE, 0.0, 1.0,
                                          (simsignaltype==3)?VAL_AUTOSCALE:VAL_MANUAL, spectrumlinmin, spectrumlinmax);
                            
                            break;
                        case 1:
                            for (i=0; i<(points/2); i++) {
                                spectrum[i] = 20*(log10(spectrum[i]));
                            }
                            SetAxisRange (mainpnl, MAINPNL_FREQGRAPH, VAL_NO_CHANGE, 0.0, 1.0,
                                          (simsignaltype==3)?VAL_AUTOSCALE:VAL_MANUAL, spectrumdbmin, spectrumdbmax);
                            
                            break;
                        case 2:
                            for (i=0; i<(points/2); i++) {
                                spectrum[i] = 20*(log10(spectrum[i]/.001));
                            }
                            SetAxisRange (mainpnl, MAINPNL_FREQGRAPH, VAL_NO_CHANGE, 0.0, 1.0,
                                          (simsignaltype==3)?VAL_AUTOSCALE:VAL_MANUAL, spectrumdbmmin, spectrumdbmmax);
                            
                            break;
                    }
                    PlotWaveform (mainpnl, MAINPNL_FREQGRAPH, spectrum, points/2,
                                  VAL_DOUBLE, 1.0, 0.0, 0.0, freqspace, VAL_THIN_LINE,
                                  VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
                }
                else {
                   SetAxisRange (mainpnl, MAINPNL_FREQGRAPH, VAL_AUTOSCALE, 0.0, 1.0,
                         VAL_MANUAL, timewavemin, timewavemax);
           
                    PlotY (mainpnl, MAINPNL_FREQGRAPH, filteredwave, points, VAL_DOUBLE,
                           VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
                }
                
            }  /* END OF ELSE */



            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Subroutine to enable or disable controls on the Filter Panel */
/*                                                               */
/*****************************************************************/
void SetFilterPnlInputModes(char * charArray) 
{
    SetInputMode (filterpnl, FILTERPNL_FILTTYPE,   charArray[ 0]=='1');
    SetInputMode (filterpnl, FILTERPNL_IIRTYPE,    charArray[ 1]=='1');
    SetInputMode (filterpnl, FILTERPNL_IIRORDER,   charArray[ 2]=='1');
    SetInputMode (filterpnl, FILTERPNL_IIRCUTOFF1, charArray[ 3]=='1');
    SetInputMode (filterpnl, FILTERPNL_IIRCUTOFF2, charArray[ 4]=='1');
    SetInputMode (filterpnl, FILTERPNL_FILTERFUNC, charArray[ 5]=='1');
    SetInputMode (filterpnl, FILTERPNL_PASSBNDRIP, charArray[ 6]=='1');
    SetInputMode (filterpnl, FILTERPNL_STPBNDATT,  charArray[ 7]=='1');
    SetInputMode (filterpnl, FILTERPNL_FIRFILTYPE, charArray[ 8]=='1');
    SetInputMode (filterpnl, FILTERPNL_FIRWINDOW,  charArray[ 9]=='1');
    SetInputMode (filterpnl, FILTERPNL_FIRLENGTH,  charArray[10]=='1');
    SetInputMode (filterpnl, FILTERPNL_FIRCUTOFF1, charArray[11]=='1');
    SetInputMode (filterpnl, FILTERPNL_FIRCUTOFF2, charArray[12]=='1');
    SetInputMode (filterpnl, FILTERPNL_KAISERBETA, charArray[13]=='1');
}               

/*****************************************************************/
/*                                                               */
/*  Enablefilters Function Description:                          */
/*                                                               */
/*  The Enablefilters function is called whenever the user       */
/*  clicks on the filter enable/disable switch on the filter     */
/*  selection panel.  If the user left-clicks on the control,    */
/*  the function gets the value from the control and places it   */
/*  in the filterenable variable, which indicates whether or not */
/*  filtering of the data is enabled or disabled.  If filtering  */
/*  is disabled, then all the filter parameter input controls    */
/*  are disabled and dim on the filter selection panel.  If      */
/*  filtering is enabled, then the series of switch statements   */
/*  determine which filter has been selected and which filter    */
/*  parameter input controls should be enabled or disabled.  If  */
/*  the user right-clicks on the control, this function displays */
/*  a help message with information about the filter enable      */
/*  control.                                                     */
/*                                                               */
/*****************************************************************/


int CVICALLBACK Enablefilters (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_FILTERENABLE, &filterenable);
            if (filterenable == 0) 
                SetFilterPnlInputModes("00000000000000");
            else if ((filterenable == 1) && (initial == 1))
            {
                initial = 0;
                SetFilterPnlInputModes("10000100111100");
            }
            else
            {
                switch (filttype) {
                    case 0:
                        switch (firfiltype) {
                            case 0:
                                switch (filterfunc) {
                                    case 0:
                                        SetFilterPnlInputModes("10000100111100");
                                        break;
                                    case 1:
                                        SetFilterPnlInputModes("10000100111100");
                                        break;
                                    case 2:
                                        SetFilterPnlInputModes("10000100111110");
                                        break;
                                    case 3:
                                        SetFilterPnlInputModes("10000100111110");
                                        break;
                                }
                                
                                break;
                            case 1:
                                switch (filterfunc) {
                                    case 0:
                                        SetFilterPnlInputModes("10000100101101");
                                        break;
                                    case 1:
                                        SetFilterPnlInputModes("10000100101101");
                                        break;
                                    case 2:
                                        SetFilterPnlInputModes("10000100101111");
                                        break;
                                    case 3:
                                        SetFilterPnlInputModes("10000100101111");
                                        break;
                                }
                                break;
                        }
                        
                        break;
                    case 1:
                        switch (iirtype) {
                            case 0:
                                switch (filterfunc) {
                                    case 0:
                                        SetFilterPnlInputModes("11110100000000");
                                        break;
                                    case 1:
                                        SetFilterPnlInputModes("11110100000000");
                                        break;
                                    case 2:
                                        SetFilterPnlInputModes("11111100000000");
                                        break;
                                    case 3:
                                        SetFilterPnlInputModes("11111100000000");
                                        break;
                                }
                                
                                break;
                            case 1:
                                switch (filterfunc) {
                                    case 0:
                                        SetFilterPnlInputModes("11110110000000");
                                        break;
                                    case 1:
                                        SetFilterPnlInputModes("11110110000000");
                                        break;
                                    case 2:
                                        SetFilterPnlInputModes("11111110000000");
                                        break;
                                    case 3:
                                        SetFilterPnlInputModes("11111110000000");
                                        break;
                                }
                                
                                break;
                            case 2:
                                switch (filterfunc) {
                                    case 0:
                                        SetFilterPnlInputModes("11110101000000");
                                        break;
                                    case 1:
                                        SetFilterPnlInputModes("11110101000000");
                                        break;
                                    case 2:
                                        SetFilterPnlInputModes("11111101000000");
                                        break;
                                    case 3:
                                        SetFilterPnlInputModes("11111101000000");
                                        break;
                                }
                                
                                break;
                            case 3:
                                switch (filterfunc) {
                                    case 0:
                                        SetFilterPnlInputModes("11110111000000");
                                        break;
                                    case 1:
                                        SetFilterPnlInputModes("11110111000000");
                                        break;
                                    case 2:
                                        SetFilterPnlInputModes("11111111000000");
                                        break;
                                    case 3:
                                        SetFilterPnlInputModes("11111111000000");
                                        break;
                                }
                                break;
                        }
                        break;
                }
             }   
            break;
    case EVENT_RIGHT_CLICK:
        MessagePopup ("Enable/Disable Filters",
                      "Enables or disables filtering of the acquired signal.");
        break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getfiltype Function Description:                             */
/*                                                               */
/*  The Getfiltype function is called whenever the user clicks   */
/*  on the FIR/IIR filter type selector.  If the user left-      */
/*  clicks on the control, the function gets the value from the  */
/*  filter type control and puts in the filttype variable.  A    */
/*  series of switch statements similar to those in the above    */
/*  Enablefilters function determines which filter type has been */
/*  selected and enables and disables the appropriate filter     */
/*  input controls.  If the user right-clicks on the control,    */
/*  this function displays a help message with information about */
/*  the control.                                                 */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getfiltype (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_FILTTYPE, &filttype);
            
            switch (filttype) {
                    case 0:  /* FIR */
                        switch (firfiltype) {
                            case 0:  /* WINDOW */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 17;  /* LP WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111100");
                                        break;
                                    case 1:  /* HIGHPASS */
                                        filtercode = 18; /* HP WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111100");
                                        break;
                                    case 2:   /* BANDPASS */
                                        filtercode = 19;  /* BP WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111110");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 20; /* BS WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111110");
                                        break;
                                }
                                
                                break;
                            case 1: /* KAISER */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 21; /* LP KAISER FIR */
                                        SetFilterPnlInputModes("10000100101101");
                                        break;
                                    case 1:  /* HIGHPASS */
                                        filtercode = 22; /* HP KAISER FIR */
                                        SetFilterPnlInputModes("10000100101101");
                                        break;
                                    case 2: /* BANDPASS */
                                        filtercode = 23; /* BP KAISER FIR */
                                        SetFilterPnlInputModes("10000100101111");
                                        break;
                                    case 3: /* BANDSTOP */
                                        filtercode = 24; /* BS KAISER FIR */
                                        SetFilterPnlInputModes("10000100101111");
                                        break;
                                }
                                break;
                        }
                        
                        break;
                    case 1:  /* IIR */
                        switch (iirtype) {
                            case 0: /* BUTTERWORTH */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 1; /* LP BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11110100000000");
                                        break;
                                    case 1:  /* HIGHPASS */ 
                                        filtercode = 2;  /* HP BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11110100000000");
                                        break;
                                    case 2: /* BANDPASS */
                                        filtercode = 3; /* BP BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11111100000000");
                                        break;
                                    case 3:  /* BANDSTOP */
                                        filtercode = 4; /* BS BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11111100000000");
                                        break;
                                }
                                
                                break;
                            case 1: /* CHEBYSHEV */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 5; /* LP CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11110110000000");
                                        break;
                                    case 1:   /* HIGHPASS */
                                        filtercode = 6; /* HP CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11110110000000");
                                        break;
                                    case 2:  /* BANDPASS */
                                        filtercode = 7; /* BP CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11111110000000");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 8;  /* BS CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11111110000000");
                                        break;
                                }
                                
                                break;
                            case 2:  /* INV CHEBYSHEV */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 9; /* LP INV CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11110101000000");
                                        break;
                                    case 1:  /* HIGHPASS */
                                        filtercode = 10; /* HP INV CHEBYSHEV */
                                        SetFilterPnlInputModes("11110101000000");
                                        break;
                                    case 2:  /* BANDPASS */
                                        filtercode = 11; /* BP INVERSE CHEBYSHEV */
                                        SetFilterPnlInputModes("11111101000000");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 12; /* BS INV CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11111101000000");
                                        break;
                                }
                                
                                break;
                            case 3:  /* ELLIPTIC */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 13; /* LP ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11110111000000");
                                        break;
                                    case 1: /* HIGHPASS */
                                        filtercode = 14; /* HP ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11110111000000");
                                        break;
                                    case 2:  /* BANDPASS */
                                        filtercode = 15; /* BP ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11111111000000");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 16; /* BS ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11111111000000");
                                        break;
                                } /* END OF SWITCH ON FILTERFUNC */
                                
                                break;
                        }   /* END OF SWITCH ON IIRTYPE */
                        
                        break;
                }   /* END OF SWITCH ON FILTTYPE */
            break;
    case EVENT_RIGHT_CLICK:
        MessagePopup ("Select Filter Type",
                      "Selects between IIR and FIR filters.");
        break;
    } /* END OF SWITCH ON COMMIT */
    return 0;
} /* END OF FUNCTION */

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getiirtype Function Description:                             */
/*                                                               */
/*  The Getiirtype function is called when the user clicks on    */
/*  IIR filter type selector.  The four type of IIR filters      */
/*  available are Butterworth, Chebyshev, Inverse Chebyshev, and */
/*  Elliptic.  If the user left-clicks on the control, the       */
/*  function gets the value from the control and puts it in the  */
/*  iirtype variable.  A series of switch statements determines  */
/*  which input controls should be enabled or disabled.  If the  */
/*  user right-clicks on the control, a help message with        */
/*  information about the control is displayed.                  */
/*                                                               */
/*****************************************************************/


int CVICALLBACK Getiirtype (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_IIRTYPE, &iirtype);
            switch (iirtype) {
                            case 0: /* BUTTERWORTH */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 1; /* LP BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11110100000000");
                                        break;
                                    case 1:  /* HIGHPASS */ 
                                        filtercode = 2;  /* HP BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11110100000000");
                                        break;
                                    case 2: /* BANDPASS */
                                        filtercode = 3; /* BP BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11111100000000");
                                        break;
                                    case 3:  /* BANDSTOP */
                                        filtercode = 4; /* BS BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11111100000000");
                                        break;
                                }
                                
                                break;
                            case 1: /* CHEBYSHEV */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 5; /* LP CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11110110000000");
                                        break;
                                    case 1:   /* HIGHPASS */
                                        filtercode = 6; /* HP CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11110110000000");
                                        break;
                                    case 2:  /* BANDPASS */
                                        filtercode = 7; /* BP CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11111110000000");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 8;  /* BS CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11111110000000");
                                        break;
                                }
                                
                                break;
                            case 2:  /* INV CHEBYSHEV */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 9; /* LP INV CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11110101000000");
                                        break;
                                    case 1:  /* HIGHPASS */
                                        filtercode = 10; /* HP INV CHEBYSHEV */
                                        SetFilterPnlInputModes("11110101000000");
                                        break;
                                    case 2:  /* BANDPASS */
                                        filtercode = 11; /* BP INVERSE CHEBYSHEV */
                                        SetFilterPnlInputModes("11111101000000");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 12; /* BS INV CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11111101000000");
                                        break;
                                }
                                
                                break;
                            case 3:  /* ELLIPTIC */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 13; /* LP ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11110111000000");
                                        break;
                                    case 1: /* HIGHPASS */
                                        filtercode = 14; /* HP ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11110111000000");
                                        break;
                                    case 2:  /* BANDPASS */
                                        filtercode = 15; /* BP ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11111111000000");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 16; /* BS ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11111111000000");
                                        break;
                                } /* END OF SWITCH ON FILTERFUNC */
                                
                                break;
                        }   /* END OF SWITCH ON IIRTYPE */
                        
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select IIR Filter Type",
                          "Select between Butterworth, Chebyshev, Inverse Chebyshev, or Elliptic IIR Filters.");
            break;
    }  /* END OF SWITCH ON COMMIT */
    return 0;
}  /* END OF FUNCTION */

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Make sure Lower Cutoff is greater than Upper Cutoff          */
/*                                                               */
/*****************************************************************/

void CheckCutoffs (void)
{
    if ((filterenable == 1) && (filttype == 0) && ((filterfunc ==2) || (filterfunc==3))) {
        if (fircutoff2 <= fircutoff1) {
           MessagePopup ("FIR Cutoff Frequency Error",
                         "The upper cutoff frequency must be greater than the lower cutoff frequency.  The upper cutoff frequency has been reset to 100 Hz greater than the lower cutoff frequency.");
           if (fircutoff1 == (samprate/2)) {
              fircutoff1 = fircutoff1 - 100;
              SetCtrlVal (filterpnl, FILTERPNL_FIRCUTOFF1, fircutoff1);
              }
           fircutoff2 = fircutoff1 + 100;
           SetCtrlVal (filterpnl, FILTERPNL_FIRCUTOFF2, fircutoff2);
           }
    }
    if ((filterenable == 1) && (filttype == 1) && ((filterfunc ==2) || (filterfunc==3))) {
        if (iircutoff2 <= iircutoff1) {
           MessagePopup ("IIR Cutoff Frequency Error",
                         "The upper cutoff frequency must be greater than the lower cutoff frequency.  The upper cutoff frequency has been reset to 100 Hz greater than the lower cutoff frequency.");
           if (iircutoff1 == (samprate/2)) {
              iircutoff1 = iircutoff1 - 100;
              SetCtrlVal (filterpnl, FILTERPNL_IIRCUTOFF1, iircutoff1);
              }
           iircutoff2 = iircutoff1 + 100;
           SetCtrlVal (filterpnl, FILTERPNL_IIRCUTOFF2, iircutoff2);
           }
    }
}   

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getiirorder Function Description:                            */
/*                                                               */
/*  The Getiirorder function is called when the user clicks on   */
/*  the iirorder input control.  If the user changes the value,  */
/*  the function gets the value from the control and puts it in  */
/*  the iirorder variable.  If the user right-clicks on the      */
/*  control, a help message is displayed containing information  */
/*  about the control.                                           */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getiirorder (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_IIRORDER, &iirorder);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set IIR Filter Order",
                          "Sets the order of the IIR Filter.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getfircutoff1 Function Description:                          */
/*                                                               */
/*  The Getfircutoff1 function is called whenever the user       */
/*  clicks on the FIR lower cutoff frequency control.  Whenever  */
/*  the user changes the value of the control, the function gets */
/*  the value from the control and puts it in the fircutoff1     */
/*  variable.  If the user right-clicks on the control, the      */
/*  function displays a help message with information about the  */
/*  control.                                                     */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getfircutoff1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_FIRCUTOFF1, &fircutoff1);
            CheckCutoffs();
            
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Lower FIR Cutoff Frequency",
                          "Sets the lower cutoff frequency of the FIR filter (in Hertz). This is the only cutoff frequency for Lowpass and Highpass filters.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getiircutoff1 Function Description:                          */
/*                                                               */
/*  The Getiircutoff1 function is called when the user clicks on */
/*  the IIR lower cutoff frequency input control.  If the user   */
/*  changes the value of the control, the function gets the new  */
/*  value of the control and puts it in the iircutoff1 variable. */
/*  If the user right-clicks on the control, a help message is   */
/*  displayed with information about the control.                */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getiircutoff1 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_IIRCUTOFF1, &iircutoff1);
            CheckCutoffs();
            
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Lower IIR Cutoff Frequency",
                          "Sets the lower cutoff frequency of the IIR filter (in Hertz).  This is the only cutoff frequency for Lowpass and Highpass filters.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getfircutoff2 Function Description:                          */
/*                                                               */
/*  The Getfircutoff2 function is called whenever the user       */
/*  clicks on the FIR upper cutoff frequency control.  If the    */
/*  user changes the control value, the function gets the new    */
/*  value from the control and puts it in the fircutoff2         */
/*  variable.  If the user right-clicks on the control, a help   */
/*  message is displayed with information about the control.     */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getfircutoff2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_FIRCUTOFF2, &fircutoff2);
            CheckCutoffs();

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Upper FIR Cutoff Frequency",
                          "Sets the upper cutoff frequency of the FIR filter (in Hertz).  This is used only with Bandpass and Bandstop filters.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getiircutoff2 Function Description:                          */
/*                                                               */
/*  The Getiircutoff2 function is called when the user clicks on */
/*  the IIR upper cutoff frequency control.  If the user changes */
/*  the value of the control, the function gets the new value    */
/*  from the control and puts it in the iircutoff2 variable.  If */
/*  the user right-clicks on the control, the function displays  */
/*  a help message with information about the control.           */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getiircutoff2 (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_IIRCUTOFF2, &iircutoff2);
            CheckCutoffs();
            
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Upper IIR Cutoff Frequency",
                      "Sets the upper cutoff frequency of the IIR filter (in Hertz).  This is used only with Bandpass and Bandstop filters.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  The Getpassbndrip function is called when the user clicks on */
/*  the ripple control on the filter selection panel.  If the    */
/*  user changes the value of the control, the function gets the */
/*  new value of the control and puts it in the passbndrip       */
/*  variable.  If the user right-clicks on the control, the      */
/*  function displays a help message with information about the  */
/*  control.                                                     */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getpassbndrip (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_PASSBNDRIP, &passbndrip);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select Ripple",
                          "Sets the value (in dB) of ripple in the passband for Chebyshev and Elliptic filters." );
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getstpbndatt Function Description:                           */
/*                                                               */
/*  The Getstpbndatt function is called when the user clicks on  */
/*  the stop band attenuation control on the filter selection    */
/*  panel.  If the user changes the value of the control, the    */
/*  function gets the new value of the control and puts it in    */
/*  the stpbndatt variable.  If the user right-clicks on the     */
/*  control, the function displays a help message with           */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getstpbndatt (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_STPBNDATT, &stpbndatt);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select Attenuation",
                          "Sets the value (in dB) of attenuation in the stopband for Inverse Chebyshev and Elliptic filters.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getfirfiltype Function Description:                          */
/*                                                               */
/*  The Getfirfiltype function is called when the user clicks on */
/*  the Window/Kaiser FIR filter type selector.  If the user     */
/*  changes the value of the switch, the function gets the new   */
/*  value from the control and puts it in the firfiltype         */
/*  variable.  A series of switch functions determine which      */
/*  filter input controls to enable or disable.  If the user     */
/*  right-clicks on the control, the function displays a help    */
/*  message with information about the control.                  */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getfirfiltype (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_FIRFILTYPE, &firfiltype);
            switch (firfiltype) {
                            case 0:  /* WINDOW */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 17;  /* LP WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111100");
                                        break;
                                    case 1:  /* HIGHPASS */
                                        filtercode = 18; /* HP WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111100");
                                        break;
                                    case 2:   /* BANDPASS */
                                        filtercode = 19;  /* BP WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111110");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 20; /* BS WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111110");
                                        break;
                                }
                                
                                break;
                            case 1: /* KAISER */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 21; /* LP KAISER FIR */
                                        SetFilterPnlInputModes("10000100101101");
                                        break;
                                    case 1:  /* HIGHPASS */
                                        filtercode = 22; /* HP KAISER FIR */
                                        SetFilterPnlInputModes("10000100101101");
                                        break;
                                    case 2: /* BANDPASS */
                                        filtercode = 23; /* BP KAISER FIR */
                                        SetFilterPnlInputModes("10000100101111");
                                        break;
                                    case 3: /* BANDSTOP */
                                        filtercode = 24; /* BS KAISER FIR */
                                        SetFilterPnlInputModes("10000100101111");
                                        break;
                                }
                                break;
                        } /* END OF SWITCH ON FIRFILTYPE */
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select FIR Filter Type",
                          "Sets type of FIR filter used (Window or Kaiser methods).");
            break;
    }  /* END OF SWITCH ON COMMIT */
    return 0;
}  /* END OF FUNCTION */

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getfirwindow Function Description:                           */
/*                                                               */
/*  The Getfirwindow function is called when the user clicks on  */
/*  the FIR filter window type selector.  There are five         */
/*  available windows:  rectangular, triangular, Hanning,        */
/*  Hamming, and Blackman.  If the user changes the value of the */
/*  control, the function gets the new value from the contol and */
/*  puts it in the firwindow variable.  If the user right-clicks */
/*  on the control, the function displays a help message with    */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getfirwindow (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_FIRWINDOW, &firwindow);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select FIR Window",
                          "Selects the window used in the FIR filter (used in window mode only).");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getfirlength Function Description:                           */
/*                                                               */
/*  The Getfirlength function is called when the user clicks on  */
/*  the FIR filter length control on the filter selection panel. */
/*  If the user changes the value of the control, the function   */
/*  gets the new value of the control and puts it in the         */
/*  firlength variable.  If the user right-clicks on the control */
/*  the function displays a help message with information about  */
/*  the control.                                                 */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getfirlength (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_FIRLENGTH, &firlength);
            
            /* Force value to be odd for some FIR functions */
            if (!(firlength%2)) {
                firlength++;
                SetCtrlVal (filterpnl, FILTERPNL_FIRLENGTH, firlength);
            }    

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select FIR Filter Length",
                          "Sets the number of points for the FIR filter(Odd value between 1-79).");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getkaibeta Function Description:                             */
/*                                                               */
/*  The Getkaibeta function is called when the user clicks on    */
/*  the Kaiser beta parameter control on the filter selection    */
/*  panel.  If the user changes the value of the control, the    */
/*  function gets the new value of the control and puts it in    */
/*  the kaiserbeta variable.  If the user right-clicks on the    */
/*  control, the function displays a help message with           */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getkaibeta (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_KAISERBETA, &kaiserbeta);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select Beta",
                          "Sets the value for the beta parameter in the Kaiser window.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getfilterfunc Function Description:                          */
/*                                                               */
/*  The Getfilterfunc function is called when the user clicks on */
/*  the filter function control on the filter selection panel.   */
/*  If the user changes the value of the control, the function   */
/*  gets the new value of the control and puts it in the         */
/*  filterfunc variable.  A series of switch statements          */
/*  determine which filter input controls to enable or disable.  */
/*  If the user right-clicks on the control, the function        */
/*  displays a help message with information about the control.  */
/*                                                               */
/*****************************************************************/


int CVICALLBACK Getfilterfunc (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (filterpnl, FILTERPNL_FILTERFUNC, &filterfunc);
            switch (filttype) {
                    case 0:  /* FIR */
                        switch (firfiltype) {
                            case 0:  /* WINDOW */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 17;  /* LP WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111100");
                                        break;
                                    case 1:  /* HIGHPASS */
                                        filtercode = 18; /* HP WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111100");
                                        break;
                                    case 2:   /* BANDPASS */
                                        filtercode = 19;  /* BP WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111110");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 20; /* BS WINDOW FIR */
                                        SetFilterPnlInputModes("10000100111110");
                                        break;
                                }
                                
                                break;
                            case 1: /* KAISER */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 21; /* LP KAISER FIR */
                                        SetFilterPnlInputModes("10000100101101");
                                        break;
                                    case 1:  /* HIGHPASS */
                                        filtercode = 22; /* HP KAISER FIR */
                                        SetFilterPnlInputModes("10000100101101");
                                        break;
                                    case 2: /* BANDPASS */
                                        filtercode = 23; /* BP KAISER FIR */
                                        SetFilterPnlInputModes("10000100101111");
                                        break;
                                    case 3: /* BANDSTOP */
                                        filtercode = 24; /* BS KAISER FIR */
                                        SetFilterPnlInputModes("10000100101111");
                                        break;
                                }
                                break;
                        }
                        
                        break;
                    case 1:  /* IIR */
                        switch (iirtype) {
                            case 0: /* BUTTERWORTH */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 1; /* LP BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11110100000000");
                                        break;
                                    case 1:  /* HIGHPASS */ 
                                        filtercode = 2;  /* HP BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11110100000000");
                                        break;
                                    case 2: /* BANDPASS */
                                        filtercode = 3; /* BP BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11111100000000");
                                        break;
                                    case 3:  /* BANDSTOP */
                                        filtercode = 4; /* BS BUTTERWORTH IIR */
                                        SetFilterPnlInputModes("11111100000000");
                                        break;
                                }
                                
                                break;
                            case 1: /* CHEBYSHEV */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 5; /* LP CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11110110000000");
                                        break;
                                    case 1:   /* HIGHPASS */
                                        filtercode = 6; /* HP CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11110110000000");
                                        break;
                                    case 2:  /* BANDPASS */
                                        filtercode = 7; /* BP CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11111110000000");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 8;  /* BS CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11111110000000");
                                        break;
                                }
                                
                                break;
                            case 2:  /* INV CHEBYSHEV */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 9; /* LP INV CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11110101000000");
                                        break;
                                    case 1:  /* HIGHPASS */
                                        filtercode = 10; /* HP INV CHEBYSHEV */
                                        SetFilterPnlInputModes("11110101000000");
                                        break;
                                    case 2:  /* BANDPASS */
                                        filtercode = 11; /* BP INVERSE CHEBYSHEV */
                                        SetFilterPnlInputModes("11111101000000");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 12; /* BS INV CHEBYSHEV IIR */
                                        SetFilterPnlInputModes("11111101000000");
                                        break;
                                }
                                
                                break;
                            case 3:  /* ELLIPTIC */
                                switch (filterfunc) {
                                    case 0: /* LOWPASS */
                                        filtercode = 13; /* LP ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11110111000000");
                                        break;
                                    case 1: /* HIGHPASS */
                                        filtercode = 14; /* HP ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11110111000000");
                                        break;
                                    case 2:  /* BANDPASS */
                                        filtercode = 15; /* BP ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11111111000000");
                                        break;
                                    case 3:   /* BANDSTOP */
                                        filtercode = 16; /* BS ELLIPTIC IIR */
                                        SetFilterPnlInputModes("11111111000000");
                                        break;
                                } /* END OF SWITCH ON FILTERFUNC */
                                
                                break;
                        }   /* END OF SWITCH ON IIRTYPE */
                        
                        break;
                }   /* END OF SWITCH ON FILTTYPE */

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select Filter Function",
                          "Selects between lowpass, highpass, bandpass, and bandstop filters.");
            break;
    }
    return 0;
}

/*****************************************************************/
/*                                                               */
/*  Returntomain Function Description:                           */
/*                                                               */
/*  The Returntomain function is called when the user clicks on  */
/*  the OK button to return to the main panel from the filter    */
/*  selection panel.  If the user left-clicks on the button, the */
/*  function first checks for valid values for the upper FIR and */
/*  IIR cutoff frequencies.  If the upper cutoff frequency is    */
/*  less than the lower cutoff frequency, the program sets the   */
/*  upper cutoff frequency to be 100 Hz greater than the lower   */
/*  cutoff frequency, except in the case where the lower cutoff  */
/*  frequency is already set to 1/2 the sampling frequency.  In  */
/*  this case the lower cutoff frequency is reduced by 100 Hz    */
/*  and the upper cutoff frequency is set to 1/2 the sampling    */
/*  rate.  A message pops up to inform the user of these changes */
/*  if they do occur.  The function then closes the filter       */
/*  selection panel and returns to the main panel.  If the       */
/*  system had been acquiring data before the Filters button had */
/*  been pressed, the timer is enabled and data acquisition      */
/*  resumes.  If the user right-clicks on the OK button, the     */
/*  function displays a help message with information about the  */
/*  control.                                                     */
/*                                                               */
/*****************************************************************/


int CVICALLBACK Returntomain (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (filterpnl);
            
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Return to Main Screen",
                          "Applies the chosen filter parameters and returns to the data acquisition screen.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  ConfigureDaq Function Description:                           */
/*                                                               */
/*  The ConfigureDaq function is called when the user clicks on  */
/*  the DAQ Configuration button on the main panel.  If the user */
/*  left-clicks on the button, the function displays the DAQ     */
/*  configuration panel and disables the timer and data          */
/*  acquisition.  If the user right-clicks on the DAQ            */
/*  Configuration button, the function displays a help message   */
/*  containing information about the control.                    */
/*                                                               */
/*****************************************************************/

int CVICALLBACK ConfigureDaq (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            
            DisplayPanel (daqpnl);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Configure DAQ", "Displays and allows user to change acquisition parameters.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  GetBoard Function Description:                               */
/*                                                               */
/*  The GetBoard function is called when the user clicks on the  */
/*  board control on the DAQ configuration panel.  If the user   */
/*  changes the value of the control, the function gets the new  */
/*  control value and puts it in the board variable.  If the     */
/*  right-clicks on the control, the function displays a help    */
/*  message with information about the control.                  */
/*                                                               */
/*****************************************************************/

int CVICALLBACK GetBoard (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (daqpnl, DAQPANEL_BOARD, &board);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Board Number",
                          "Sets the data acquisition board number.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getchannel Function Description:                             */
/*                                                               */
/*  The Getchannel function is called when the user clicks on    */
/*  the channel control on the DAQ configuration panel.  If the  */
/*  user changes the value of the control, the function gets the */
/*  new control value and puts it in the channel variable.  If   */
/*  the user right-clicks on the control, the function displays  */
/*  a help message with information about the channel control.   */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getchannel (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (daqpnl, DAQPANEL_CHANNEL, channel);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Channel Number",
                          "Sets the channel number to acquire data from.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getsamprate Function Description:                            */
/*                                                               */
/*  The Getsamprate function is called when the user clicks on   */
/*  the sampling rate control on the data acquisition            */
/*  configuration panel.  If the user changes the value of the   */
/*  sampling rate control, the function gets the new sampling    */
/*  rate from the control and puts it in the variable samprate.  */
/*  If either of the lower cutoff frequencies are set greater    */
/*  than 1/2 the new sampling rate, the cutoff frequencies are   */
/*  set to a value of 100 Hz below 1/2 the sampling rate.  These */
/*  values are stored in the variables and sent to the controls  */
/*  on the filter selection panel.  In the same way, if either   */
/*  of the upper cutoff frequencies are set greater than 1/2 the */
/*  new sampling rate, the cutoff frequencies are set to be      */
/*  equal to 1/2 the sampling rate.  The maximum possible values */
/*  for all four cutoff frequencies are set to 1/2 the sampling  */
/*  rate.  If the user right-clicks on the control, a help       */
/*  message is displayed with information about the control.     */
/*                                                               */
/*****************************************************************/


int CVICALLBACK Getsamprate (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (daqpnl, DAQPANEL_SAMPRATE, &samprate);
            if (fircutoff1 > (samprate/2)) {
               fircutoff1 = (samprate/2) - 100;
               SetCtrlVal (filterpnl, FILTERPNL_FIRCUTOFF1, fircutoff1);
               }
            if (fircutoff2 > (samprate/2)) {
               fircutoff2 = (samprate/2);
               SetCtrlVal (filterpnl, FILTERPNL_FIRCUTOFF2, fircutoff2);
               }
            if (iircutoff1 > (samprate/2)) {
               iircutoff1 = (samprate/2) - 100;
               SetCtrlVal (filterpnl, FILTERPNL_IIRCUTOFF1, iircutoff1);
               }
            if (iircutoff2 > (samprate/2)) {
               iircutoff2 = (samprate/2);
               SetCtrlVal (filterpnl, FILTERPNL_IIRCUTOFF2, iircutoff2);
               }
            SetCtrlAttribute (filterpnl, FILTERPNL_FIRCUTOFF1, ATTR_MAX_VALUE,
                              (samprate/2));
            SetCtrlAttribute (filterpnl, FILTERPNL_FIRCUTOFF2, ATTR_MAX_VALUE,
                              (samprate/2));
            SetCtrlAttribute (filterpnl, FILTERPNL_IIRCUTOFF1, ATTR_MAX_VALUE,
                              (samprate/2));
            SetCtrlAttribute (filterpnl, FILTERPNL_IIRCUTOFF2, ATTR_MAX_VALUE,
                              (samprate/2));
               
                  
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Sampling Rate",
                          "Sets the sampling rate for acquiring data.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getpoints Function Description:                              */
/*                                                               */
/*  The Getpoints function is called when the user clicks on the */
/*  points control on the DAQ configuration panel.  If the user  */
/*  changes the value of the points control, the function gets   */
/*  the new value of the control and puts it in the variable     */
/*  points.  If the user right-clicks on the control, the        */
/*  function displays a help message with information about the  */
/*  control.                                                     */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getpoints (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (daqpnl, DAQPANEL_POINTS, &points);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Number of Points",
                          "Sets the number of points to acquire and analyze with each iteration.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Quitdaqconf Function Description:                            */
/*                                                               */
/*  The Quitdaqconf function is called when the user clicks on   */
/*  the OK button on the DAQ configuration panel.  If the user   */
/*  left clicks on the button, the function closes the DAQ       */
/*  configuration panel and restarts the timer for data          */
/*  acquisition if the system was running when the user entered  */
/*  DAQ Configuration mode.  If the user right-clicks on the     */
/*  button, the function displays a help message with            */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Quitdaqconf (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (daqpnl);
            
            if (running == 1) 
            {
                
                if (signalswitch == 1) {
                   AIClearAcquisition (task);
                   AIStartAcquisition (board, channel, 2*points, samprate, 10, 0, &rate,&task);
                }
            }
            
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Return to Main Screen",
                          "Applies the chosen acquisition parameters and returns to the main screen.");
            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getdisplaymode Function Description:                         */
/*                                                               */
/*  The Getdisplaymode function is called when the user clicks   */
/*  on the graph display control on the main panel.  If the user */
/*  changes the value of the control the function gets the new   */
/*  value of the control and puts it in the variable displaymode.*/
/*  The function then changes the title and axis labels of the   */
/*  lower graphs to match the selected function, either time     */
/*  waveform or frequency spectrum.  If the user right-clicks on */
/*  the control, a help message is displayed with information    */
/*  about the control.                                           */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getdisplaymode (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (mainpnl, MAINPNL_FREQDISPLAY, &displaymode);
            if (displaymode==1) {
               SetInputMode (mainpnl, MAINPNL_SCALE, 0);
               SetCtrlAttribute (mainpnl, MAINPNL_FREQGRAPH, ATTR_XNAME, "Time");
               SetCtrlAttribute (mainpnl, MAINPNL_FREQGRAPH, ATTR_YNAME, "Volts");
               SetCtrlAttribute (mainpnl, MAINPNL_FREQGRAPH, ATTR_LABEL_TEXT,
                                 "Processed Time Waveform");
            
            }
            else {
               SetInputMode (mainpnl, MAINPNL_SCALE, 1);
               SetCtrlAttribute (mainpnl, MAINPNL_FREQGRAPH, ATTR_XNAME, "Hertz");
               SetCtrlAttribute (mainpnl, MAINPNL_FREQGRAPH, ATTR_YNAME, "Magnitude");
               SetCtrlAttribute (mainpnl, MAINPNL_FREQGRAPH, ATTR_LABEL_TEXT,
                                 "Power Spectrum");
            
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Graph Display Selection",
                          "Selects between the processed time waveform or the frequency spectrum for display on the lower graph.");

            break;
    }
    return 0;
}
/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Callhelp Function Description:                               */
/*                                                               */
/*  The Callhelp function is called when the user clicks on the  */
/*  Help button on the main panel.  If the user left-clicks on   */
/*  the button, the function displays the help panel, which      */
/*  contains general program help information. The timer is      */
/*  disabled, halting data acquisition if enabled. If the user   */
/*  right-clicks on the button, a help message is displayed with */
/*  help information about the button.                           */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Callhelp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            SetCtrlAttribute (mainpnl, MAINPNL_TIMER, ATTR_ENABLED, 0);
            
            HidePanel (mainpnl);
            DisplayPanel (helppnl);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help",
                          "Displays help and information about the program.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Quithelp Function Description:                               */
/*                                                               */
/*  The Quithelp function is called when the user clicks on the  */
/*  exit help button on the help panel.  If the user left-clicks */
/*  on the button, the function hides the help panel and         */
/*  restores the main panel.  If the system was acquiring data   */
/*  when the help button was pressed, the function enables the   */
/*  timer and data acquisition resumes.  If the user right-      */
/*  clicks on the button, a help message is displayed with       */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Quithelp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (helppnl);
            DisplayPanel (mainpnl);
            if (running == 1) 
            {
                SetCtrlAttribute (mainpnl, MAINPNL_TIMER, ATTR_ENABLED, 1);
                if (signalswitch == 1) {
                   AIClearAcquisition (task);
                   AIStartAcquisition (board, channel, 2*points, samprate, 10, 0, &rate, &task);
                }               
            }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Exit Help",
                          "Exits the help function and returns to the main panel.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Scalegraphs Function Description:                            */
/*                                                               */
/*  The Scalegraphs function is called whenever the user clicks  */
/*  on the Graph Scaling button on the main panel.  If the user  */
/*  left-clicks on the button, the function stops data           */
/*  acquisition if enabled by disabling the timer.  The function */
/*  then displays the graph scaling panel.  If the user right-   */
/*  clicks on the button, the function displays a help message   */
/*  with information about the control.                          */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Scalegraphs (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            SetCtrlAttribute (mainpnl, MAINPNL_TIMER, ATTR_ENABLED, 0);
            DisplayPanel (graphscalepnl);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Graph Scale Settings",
                          "Allows user to set y-axis values for the graphs.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getwaveformymin Function Description:                        */
/*                                                               */
/*  The Getwaveformymin function is called when the user clicks  */
/*  on the Waveform Y Min control on the graph scaling panel.    */
/*  If the user changes the value of the control, the function   */
/*  gets the new value of the control and puts it in the         */
/*  variable waveformymin.  If the user right-clicks on the      */
/*  control, the function displays a help message with           */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getwaveformymin (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_WAVEFORMYMIN, &waveformymin);
            if (waveformymin >= waveformymax) {
               waveformymax = waveformymin + 1;
               SetCtrlVal (graphscalepnl, SCALEPNL_WAVEFORMYMAX, waveformymax);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Waveform Graph Y Axis Minimum",
                          "Sets the minimum value of the y-axis on the upper graph.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getwaveformymax Function Description:                        */
/*                                                               */
/*  The Getwaveformymax function is called when the user clicks  */
/*  on the Waveform Y Max control on the graph scaling panel.    */
/*  If the user changes the value of the control, the function   */
/*  gets the new value of the control and puts it in the         */
/*  variable waveformymax.  If the user right-clicks on the      */
/*  control, the function displays a help message with           */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getwaveformymax (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_WAVEFORMYMAX, &waveformymax);
            if (waveformymax <= waveformymin) {
               waveformymax = waveformymin + 1;
               SetCtrlVal (graphscalepnl, SCALEPNL_WAVEFORMYMAX, waveformymax);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Waveform Graph Y Axis Maximum",
                          "Sets the maximum value of the y-axis on the upper graph.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getspectrumlinmin Function Description:                      */
/*                                                               */
/*  The Getspectrumlinmin function is called when the user       */
/*  clicks on the Linear Scale Y Min control on the graph        */
/*  scaling panel.  If the user changes the value of the control */
/*  the function gets the new value of the control and puts it   */
/*  in the variable spectrumlinmin.  If the user right-clicks on */
/*  the control, the function displays a help message with       */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getspectrumlinmin (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMLINMIN, &spectrumlinmin);
            if (spectrumlinmin >= spectrumlinmax) {
               spectrumlinmax = spectrumlinmin + 1;
               SetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMLINMAX, spectrumlinmax);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Linear Scale Mode Y Axis Minimum",
                          "Sets the minimum value of the y axis on the lower graph for linear scale mode.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getspectrumlinmax Function Description:                      */
/*                                                               */
/*  The Getspectrumlinmax function is called when the user       */
/*  clicks on the Linear Scale Y Max control on the graph        */
/*  scaling panel.  If the user changes the value of the control */
/*  the function gets the new value of the control and puts it   */
/*  in the variable spectrumlinmax.  If the user right-clicks on */
/*  the control, the function displays a help message with       */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getspectrumlinmax (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMLINMAX, &spectrumlinmax);
            if (spectrumlinmax <= spectrumlinmin) {
               spectrumlinmax = spectrumlinmin + 1;
               SetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMLINMAX, spectrumlinmax);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Linear Scale Mode Y Axis Maximum",
                          "Sets the maximum value of the y axis on the lower graph for linear scale mode.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getspectrumdbmin Function Description:                       */
/*                                                               */
/*  The Getspectrumdbmin function is called when the user clicks */
/*  on the dB Scale Y Min control on the graph scaling panel.    */
/*  If the user changes the value of the control, the function   */
/*  gets the new value of the contol and puts it in the variable */
/*  spectrumdbmin.  If the user right-clicks on the control, the */
/*  function displays a help message with information about the  */
/*  control.                                                     */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getspectrumdbmin (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMDBMIN, &spectrumdbmin);
            if (spectrumdbmin >= spectrumdbmax) {
                spectrumdbmax = spectrumdbmin + 1;
                SetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMDBMAX, spectrumdbmax);
            }    
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set dB Mode Y Axis Minimum",
                          "Sets the minimum value of the y-axis for dB scale mode.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getspectrumdbmax Function Description:                       */
/*                                                               */
/*  The Getspectrumdbmax function is called when the user clicks */
/*  on the dB Scale Y Max control on the graph scaling panel.    */
/*  If the user changes the value of the control, the function   */
/*  gets the new value of the control and puts it in the         */
/*  variable spectrumdbmax.  If the user right-clicks on the     */
/*  control, the function displays a help message with           */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getspectrumdbmax (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMDBMAX, &spectrumdbmax);
            if (spectrumdbmax <= spectrumdbmin) {
               spectrumdbmax = spectrumdbmin + 1;
               SetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMDBMAX, spectrumdbmax);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set dB Mode Y Axis Maximum",
                          "Sets the maximum value of the y-axis for dB scale mode.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getspectrumdbmmin Function Description:                      */
/*                                                               */
/*  The Getspectrumdbmmin function is called when the user       */
/*  clicks on the dBm Scale Y Min control on the graph scaling   */
/*  panel.  If the user changes the value of the control, the    */
/*  function gets the new value of the control and puts it in    */
/*  the variable spectrumdbmmin.  If the user right-clicks on    */
/*  control, the function displays a help message with           */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getspectrumdbmmin (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMDBMMIN, &spectrumdbmmin);
            if (spectrumdbmmin >= spectrumdbmmax) {
               spectrumdbmmax = spectrumdbmmin + 1;
               SetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMDBMMAX, spectrumdbmmax);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set dBm Mode Y Axis Minimum",
                          "Sets the minimum value of the y-axis for dBm scale mode.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Getspectrumdbmmax Function Description:                      */
/*                                                               */
/*  The Getspectrumdbmmax function is called when the user       */
/*  clicks on the dBm Scale Y Min control on the graph scaling   */
/*  panel.  If the user changes the value of the control, the    */
/*  function gets the new value of the control and puts it in    */
/*  the variable spectrumdbmmax.  If the user right-clicks on    */
/*  the control, the function displays a help message with       */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Getspectrumdbmmax (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMDBMMAX, &spectrumdbmmax);
            if (spectrumdbmmax <= spectrumdbmmin) {
               spectrumdbmmax = spectrumdbmmin + 1;
               SetCtrlVal (graphscalepnl, SCALEPNL_SPECTRUMDBMMAX, spectrumdbmmax);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set dBm Mode Y Axis Maximum",
                          "Sets the maximum value of the y-axis for dBm scale mode.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Gettimewavemin Function Description:                         */
/*                                                               */
/*  The Gettimewavemin function is called when the user clicks   */
/*  on the Time Waveform Y Min control on the graph scaling      */
/*  panel.  If the user changes the value of the control, the    */
/*  function gets the new value of the control and puts it in    */
/*  the variable timewavemin.  If the user right-clicks on the   */
/*  control, the function displays a help message with           */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Gettimewavemin (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_TIMEWAVEMIN, &timewavemin);
            if (timewavemin >= timewavemax) {
               timewavemax = timewavemin + 1;
               SetCtrlVal (graphscalepnl, SCALEPNL_TIMEWAVEMAX, timewavemax);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Time Waveform Mode Y Axis Minimum",
                          "Sets the minimum value of the y-axis for Time Waveform display mode.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Gettimewavemax Function Description:                         */
/*                                                               */
/*  The Gettimewavemax function is called when the user clicks   */
/*  on the Time Waveform Y Max control on the graph scaling      */
/*  panel.  If the user changes the value of the control, the    */
/*  function gets the new value of the control and puts it in    */
/*  the timewavemax variable.  If the user right-clicks on the   */
/*  control, the function displays a help message with           */
/*  information about the control.                               */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Gettimewavemax (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (graphscalepnl, SCALEPNL_TIMEWAVEMAX, &timewavemax);
            if (timewavemax <= timewavemin) {
               timewavemax = timewavemin + 1;
               SetCtrlVal (graphscalepnl, SCALEPNL_TIMEWAVEMAX, timewavemax);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Set Time Waveform Mode Y Axis Maximum",
                          "Sets the maximum value of the y-axis for Time Waveform display mode.");

            break;
    }
    return 0;
}

/*****************************************************************/

/*****************************************************************/
/*                                                               */
/*  Quitgraphscaling Function Description:                       */
/*                                                               */
/*  The Quitgraphscaling function is called when the user clicks */
/*  on the OK button on the graph scaling panel.  If the user    */
/*  left-clicks on the button, the function hides the graph      */
/*  scaling panel and returns to the main panel.  If the system  */
/*  was running (data acquisition enabled), the function enables */
/*  the timer and data acquisition resumes. If the user right-   */
/*  clicks on the control, the function displays a help message  */
/*  with information about the control.                          */
/*                                                               */
/*****************************************************************/

int CVICALLBACK Quitgraphscaling (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (graphscalepnl);
            if (running == 1) {
               SetCtrlAttribute (mainpnl, MAINPNL_TIMER, ATTR_ENABLED, 1);
            }   
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Exit Graph Scale Selection",
                          "Exits the graph scaling mode and returns to main panel.");

            break;
    }
    return 0;
}


int CVICALLBACK Getsignalswitch (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (daqpnl, DAQPANEL_SIGNALSWITCH, &signalswitch);
            if (signalswitch == 0) {
                SetInputMode (daqpnl, DAQPANEL_CHANNEL, 0);
                SetInputMode (daqpnl, DAQPANEL_SAMPRATE, 0);
                SetInputMode (daqpnl, DAQPANEL_SIMSIGNALTYPE, 1);
                SetInputMode (daqpnl, DAQPANEL_SIMSIGNALFREQ, 1);
                
            }
            else {
                SetInputMode (daqpnl, DAQPANEL_CHANNEL, 1);
                SetInputMode (daqpnl, DAQPANEL_SAMPRATE, 1);
                SetInputMode (daqpnl, DAQPANEL_SIMSIGNALTYPE, 0);
                SetInputMode (daqpnl, DAQPANEL_SIMSIGNALFREQ, 0);
                DisableBreakOnLibraryErrors ();
                errorchk = Init_DA_Brds (board, &boardcode);
                if (errorchk == 0) {
                    Set_DAQ_Device_Info (board, ND_DATA_XFER_MODE_AI, ND_UP_TO_1_DMA_CHANNEL);  
                    if (running == 1) {
                    
                       AIClearAcquisition (task);
                       AIStartAcquisition (board, channel, 2*points, samprate, 10, 0, &rate, &task);
                    }
                    
                }
                else {
                    MessagePopup ("Board Configuration Error",
                                  "The board you specified could not be configured.  Please check the board number and configuration.  The system has been reset to simulated data mode.");
                    signalswitch = 0;
                    SetCtrlVal (daqpnl, DAQPANEL_SIGNALSWITCH, signalswitch);
                    SetInputMode (daqpnl, DAQPANEL_CHANNEL, 0);
                    SetInputMode (daqpnl, DAQPANEL_SAMPRATE, 0);
                    SetInputMode (daqpnl, DAQPANEL_SIMSIGNALTYPE, 1);
                    SetInputMode (daqpnl, DAQPANEL_SIMSIGNALFREQ, 1);
                }
                
                
            }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select Data Source",
                          "Selects between simulated data and data acquired from a DAQ board.");

            break;
    }
    return 0;
}

int CVICALLBACK Getsimsignaltype (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (daqpnl, DAQPANEL_SIMSIGNALTYPE, &simsignaltype);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select Simulated Wave Type",
                          "Selects the simulated wave type.");

            break;
    }
    return 0;
}

int CVICALLBACK Getsimsignalfreq (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
        case EVENT_VAL_CHANGED:
            GetCtrlVal (daqpnl, DAQPANEL_SIMSIGNALFREQ, &simsignalfreq);

            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Select Simulated Wave Frequency",
                          "Selects the frequency of the simulated wave data.");

            break;
    }
    return 0;
}
