/***************************************************************************/
/*                                                                         */
/*                            UPDATE.C                                     */
/*                                                                         */
/*         This program can be used to determine the update rate           */
/*         of PlotY for various waveforms and buffer lengths.              */
/*                                                                         */
/***************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <utility.h>
#include <analysis.h>
#include <userint.h>
#include <stdlib.h>
#include <math.h>
#include "update.h"

#define YMAX 1.0
#define MAX_POINTS 8192
#define SAMPLE_PERIOD 2.0    /*  # of seconds in tight loop  */
#define TRUE 1
#define FALSE 0
#define GRAPH 1
#define CHART 0
#define SWEEP 0
#define CONTINUOUS 1
#define BLOCK 2

static double gain;
static double mark;
static double temp;
static int handle;
static int num_points;
static int hndl;
static int ctrl;
static int datatype;
static int iterations, i;
static int offscreen;
static int device;
static int shift;
static double a[MAX_POINTS];
static double b[MAX_POINTS];

int CVICALLBACK maincallback (int panelOrMenuBar,   int controlOrMenuItem,
                    int event, void *callbackData,  int eventData1,  int eventData2);

int main (int argc, char *argv[]) 
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    gain = 1.0;
    iterations = 0;
    offscreen = TRUE;
    handle = LoadPanel (0, "update.uir", UPDATE);
    SetCtrlAttribute (handle, UPDATE_GRAPH, ATTR_DATA_MODE, VAL_DISCARD);    /* This may make update faster */
    DataFunc (handle, UPDATE_NUM, EVENT_COMMIT, 0, 0, 0);
    DataFunc (handle, UPDATE_GAIN, EVENT_VAL_CHANGED, 0, 0, 0);
    DataFunc (handle, UPDATE_WAVEFORM, EVENT_COMMIT, 0, 0, 0);
    DeviceFunc(handle, UPDATE_DEVICE, EVENT_COMMIT, 0, 0, 0);
    SmoothFunc(handle, UPDATE_SMOOTH, EVENT_COMMIT, 0, 0, 0);
    ShiftFunc (handle, UPDATE_SHIFT, EVENT_COMMIT, 0, 0, 0);
    DisplayPanel (handle);
    mark = Timer ();
    RunUserInterface();

    return 0;
}

int CVICALLBACK UpdateCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) 
    {
    case EVENT_TIMER_TICK:
        if (device == GRAPH)
        {
            PlotY (handle, UPDATE_GRAPH, a, num_points, VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_GREEN);
            DeleteGraphPlot (handle, UPDATE_GRAPH, -1, 0);
        }
        else
        {
            PlotStripChart (handle, UPDATE_CHART, a, num_points, 0, 0, VAL_DOUBLE);
        }

        /*** Shift the array for visual effect ***/
        if (shift > 0)
        {
            Copy1D (a, num_points - shift, &b[shift]);
            Copy1D (&a[num_points - shift], shift, b);
            Copy1D (b, num_points, a);
        }
        iterations++;
        if (Timer() >= mark + SAMPLE_PERIOD)
        {
            SetCtrlVal (handle, UPDATE_RATE, (float)(iterations/SAMPLE_PERIOD));
            SetCtrlVal (handle, UPDATE_POINTS, (float) (iterations) * num_points / SAMPLE_PERIOD);
            iterations = 0;
            mark = Timer ();
        }

        break;
    }
    return 0;
}

int CVICALLBACK DataFunc(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    switch (control)
    {
        case UPDATE_GAIN:
            if (event == EVENT_VAL_CHANGED)
            {
                 temp = gain;
                 GetCtrlVal (handle, UPDATE_GAIN, &gain);
                 temp = gain/temp;
                 LinEv1D (a, num_points, temp, 0.0, a);
            }
            break;
        case UPDATE_NUM:
        case UPDATE_WAVEFORM:
            if (event == EVENT_COMMIT)
            {
                GetCtrlVal (handle, UPDATE_NUM, &num_points);
                GetCtrlVal (handle, UPDATE_WAVEFORM, &datatype);
                switch (datatype)
                {
                    case 0:
                        for (i=0; i<num_points; i++)
                            a[i] = (double)i/num_points * gain * YMAX;
                        break;
                    case 1:
                        for (i=0; i<num_points; i++)
                            a[i] = sin ((double) i*16*22/7/num_points) * gain * YMAX;
                        break;
                    case 2:
                        for (i=0; i<num_points; i++)
                            a[i] = ((rand()/23767.0) - 0.5) * gain * YMAX;
                        break;
                }
                SetCtrlVal (handle, UPDATE_RATE, 0.0f);
                SetCtrlVal (handle, UPDATE_POINTS, 0.0);
                SetAxisRange (handle, UPDATE_GRAPH, VAL_MANUAL, 0.0, num_points, VAL_MANUAL, -YMAX, YMAX);
                SetAxisRange (handle, UPDATE_CHART, VAL_MANUAL, 0.0, 1.0, VAL_MANUAL, -YMAX, YMAX);
                SetCtrlAttribute (handle, UPDATE_CHART, ATTR_POINTS_PER_SCREEN,
                              num_points);
                SetCtrlAttribute (handle, UPDATE_SHIFT, ATTR_MAX_VALUE, num_points - 1);
                GetCtrlVal (handle, UPDATE_SHIFT, &shift);
                if (shift > num_points-1)
                    shift = num_points - 1;
                SetCtrlVal (handle, UPDATE_SHIFT, shift);
            }
            break;
    }
    return (0);
}

int CVICALLBACK ScreenFunc(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
    {
        GetCtrlVal (handle, UPDATE_SCREENMODE, &offscreen);
        if (offscreen)
        {
            DeleteGraphPlot (handle, UPDATE_GRAPH, -1, 1);
            SetCtrlAttribute (handle, UPDATE_GRAPH, ATTR_REFRESH_GRAPH, FALSE);
        }
        else
            SetCtrlAttribute (handle, UPDATE_GRAPH, ATTR_REFRESH_GRAPH, TRUE);
    }
    return (0);
}

int CVICALLBACK DeviceFunc(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
    {
        GetCtrlVal (handle, UPDATE_DEVICE, &device);
        if (device == GRAPH)
        {
            SetAxisRange (handle, UPDATE_GRAPH, VAL_MANUAL, 0.0, num_points, VAL_MANUAL, -YMAX, YMAX);
            SetCtrlAttribute (handle, UPDATE_CHART, ATTR_VISIBLE, 0);
            SetCtrlAttribute (handle, UPDATE_CHART_MODE, ATTR_VISIBLE, 0);
            SetCtrlAttribute (handle, UPDATE_SCREENMODE, ATTR_DIMMED, 0);
            SetCtrlAttribute (handle, UPDATE_GRAPH, ATTR_VISIBLE, 1);
            SetCtrlAttribute (handle, UPDATE_SMOOTH, ATTR_VISIBLE, 1);
        }
        else
        {
            SetAxisRange (handle, UPDATE_CHART, VAL_MANUAL, 0.0, 1.0, VAL_MANUAL, -YMAX, YMAX);
            SetCtrlAttribute (handle, UPDATE_CHART, ATTR_POINTS_PER_SCREEN,
                              num_points+1);

            ChartModeFunc(handle, UPDATE_CHART_MODE, EVENT_COMMIT, 0, 0, 0);
            SetCtrlAttribute (handle, UPDATE_GRAPH, ATTR_VISIBLE, 0);
            SetCtrlAttribute (handle, UPDATE_SMOOTH, ATTR_VISIBLE, 0);
            SetCtrlAttribute (handle, UPDATE_SCREENMODE, ATTR_DIMMED, 1);
            SetCtrlAttribute (handle, UPDATE_CHART, ATTR_VISIBLE, 1);
            SetCtrlAttribute (handle, UPDATE_CHART_MODE, ATTR_VISIBLE, 1);
        }
    }
    return (0);
}

int CVICALLBACK ChartModeFunc(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int chart_mode;

    if (event == EVENT_COMMIT)
    {
        GetCtrlVal (handle, UPDATE_CHART_MODE, &chart_mode);
        switch (chart_mode)
        {
            case SWEEP:
                SetCtrlAttribute (handle, UPDATE_CHART, ATTR_SCROLL_MODE, VAL_SWEEP);
                break;
            case CONTINUOUS:
                SetCtrlAttribute (handle, UPDATE_CHART, ATTR_SCROLL_MODE, VAL_CONTINUOUS);
                break;
            case BLOCK:
                SetCtrlAttribute (handle, UPDATE_CHART, ATTR_SCROLL_MODE, VAL_BLOCK);
                break;
        }
    }
    return (0);
}

int CVICALLBACK SmoothFunc(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int smooth;

    if (event == EVENT_COMMIT)
    {
        GetCtrlVal (handle, UPDATE_SMOOTH, &smooth);
        SetCtrlAttribute (handle, UPDATE_GRAPH, ATTR_SMOOTH_UPDATE, smooth);
    }
    return (0);
}


int CVICALLBACK ShiftFunc(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{

    if (event == EVENT_VAL_CHANGED)
    {
        GetCtrlVal (handle, UPDATE_SHIFT, &shift);
    }
    return (0);
}

int CVICALLBACK QuitFunc(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
    {
        QuitUserInterface(0);
        exit (0);
    }
    return(0);
}
